@@ -2,6 +2,7 @@
 ** wacusb.c
 **
 ** Copyright (C) 2002 - 2004 - John E. Joganic
+** Copyright (C) 2003 - 2005 - Ping Cheng
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -157,22 +158,29 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 	static USBSUBTYPE xGraphire2[] =
 	{
 		{ "ET_0405", "Wacom Graphire2 4x5", 1, 0x11 },
-		{ "ET_0507", "Wacom Graphire2 5x7", 1, 0x12 },
+		{ "ET_0507", "Wacom Graphire2 5x7", 2, 0x12 },
 		{ NULL }
 	};
 
 	static USBSUBTYPE xGraphire3[] =
 	{
 		{ "ET_0405", "Wacom Graphire3 4x5", 1, 0x13 },
-		{ "ET_0608", "Wacom Graphire3 6x8", 1, 0x14 },
+		{ "ET_0608", "Wacom Graphire3 6x8", 2, 0x14 },
+		{ NULL }
+	};
+
+	static USBSUBTYPE xGraphire4[] =
+	{
+		{ "CTE_440", "Wacom Graphire4 4x5", 1, 0x15 },
+		{ "CTE_640", "Wacom Graphire4 6x8", 2, 0x16 },
 		{ NULL }
 	};
 
 	static USBSUBTYPE xIntuos[] =
 	{
-		{ "GD_0405-U", "Wacom Intuos 4x5", 1, 0x20 },
-		{ "GD_0608-U", "Wacom Intuos 6x8", 2, 0x21 },
-		{ "GD_0912-U", "Wacom Intuos 9x12", 3, 0x22 },
+		{ "GD_0405-U", "Wacom Intuos 4x5",   1, 0x20 },
+		{ "GD_0608-U", "Wacom Intuos 6x8",   2, 0x21 },
+		{ "GD_0912-U", "Wacom Intuos 9x12",  3, 0x22 },
 		{ "GD_1212-U", "Wacom Intuos 12x12", 4, 0x23 },
 		{ "GD_1218-U", "Wacom Intuos 12x18", 5, 0x24 },
 		{ NULL }
@@ -180,12 +188,16 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 
 	static USBSUBTYPE xCintiq[] =
 	{
-		{ "MODEL_PL400", "Wacom PL400", 1, 0x30 },
-		{ "MODEL_PL500", "Wacom PL500", 2, 0x31 },
-		{ "MODEL_PL600", "Wacom PL600", 3, 0x32 },
-		{ "MODEL_PL600SX", "Wacom PL600SX", 4, 0x33 },
-		{ "MODEL_PL550", "Wacom PL550", 5, 0x34 },
-		{ "MODEL_PL800", "Wacom PL800", 6, 0x35 },
+		{ "MODEL_PL400",   "Wacom PL400",    1, 0x30 },
+		{ "MODEL_PL500",   "Wacom PL500",    2, 0x31 },
+		{ "MODEL_PL600",   "Wacom PL600",    3, 0x32 },
+		{ "MODEL_PL600SX", "Wacom PL600SX",  4, 0x33 },
+		{ "MODEL_PL550",   "Wacom PL550",    5, 0x34 },
+		{ "MODEL_PL800",   "Wacom PL800",    6, 0x35 },
+		{ "MODEL_PL700",   "Wacom PL700",    7, 0x37 },
+		{ "MODEL_PL510",   "Wacom PL510",    8, 0x38 },
+		{ "MODEL_DTU710",  "Wacom PL710",   10, 0x39 },
+		{ "MODEL_DTF720",  "Wacom DTF720",  11, 0xC0 },
 		{ NULL }
 	};
 
@@ -209,17 +221,32 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 		{ NULL }
 	};
 
+	static USBSUBTYPE xVolito2[] =
+	{
+		{ "FT-0203-U", "Wacom PenStation",  1, 0x61 },
+		{ "CTF-420-U", "Wacom Volito2 4x5", 2, 0x62 },
+		{ "CTF-220-U", "Wacom Volito2 2x3", 3, 0x63 },
+		{ NULL }
+	};
+
 	static USBSUBTYPE xCintiqPartner[] =
 	{
 		{ "PTU-600", "Wacom Cintiq Partner", 1, 0x03 },
 		{ NULL }
 	};
 
+	static USBSUBTYPE xCintiqV5[] =
+	{
+		{ "DTZ-21ux", "Wacom Cintiq 21UX", 1, 0x3F },
+		{ NULL }
+	};
+
 	static USBSUBTYPE xIntuos3[] =
 	{
-		{ "PTZ-430", "Wacom Intuos3 4x5",  1, 0xB0 },
-		{ "PTZ-630", "Wacom Intuos3 6x8",  2, 0xB1 },
-		{ "PTZ-930", "Wacom Intuos3 9x12", 3, 0xB2 },
+		{ "PTZ-430",  "Wacom Intuos3 4x5",  1, 0xB0 },
+		{ "PTZ-630",  "Wacom Intuos3 6x8",  2, 0xB1 },
+		{ "PTZ-930",  "Wacom Intuos3 9x12", 3, 0xB2 },
+		{ "PTZ-631W", "Wacom Intuos3 6x8",  4, 0xB5 },
 		{ NULL }
 	};
 
@@ -229,12 +256,15 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 		{ "gr", "Graphire", WACOMDEVICE_GRAPHIRE, xGraphire },
 		{ "gr2", "Graphire2", WACOMDEVICE_GRAPHIRE2, xGraphire2 },
 		{ "gr3", "Graphire3", WACOMDEVICE_GRAPHIRE3, xGraphire3 },
+		{ "gr4", "Graphire4", WACOMDEVICE_GRAPHIRE4, xGraphire4 },
 		{ "int", "Intuos", WACOMDEVICE_INTUOS, xIntuos },
 		{ "int2", "Intuos2", WACOMDEVICE_INTUOS2, xIntuos2 },
 		{ "int3", "Intuos3", WACOMDEVICE_INTUOS3, xIntuos3 },
+		{ "ctq", "Cintiq (V5)", WACOMDEVICE_CINTIQV5, xCintiqV5 },
 		{ "pl", "Cintiq (PL)", WACOMDEVICE_CINTIQ, xCintiq },
 		{ "ptu", "Cintiq Partner (PTU)", WACOMDEVICE_PTU, xCintiqPartner },
 		{ "vol", "Volito", WACOMDEVICE_VOLITO, xVolito },
+		{ "vol2", "Volito2", WACOMDEVICE_VOLITO2, xVolito2 },
 		{ NULL }
 	};
 
@@ -684,13 +714,21 @@ static int USBParseKEY(USBTABLET* pUSB, struct input_event* pEv)
 		case BTN_TOUCH: button = WACOMBUTTON_TOUCH; break;
 		case BTN_STYLUS: button = WACOMBUTTON_STYLUS; break;
 		case BTN_STYLUS2: button = WACOMBUTTON_STYLUS2; break;
+		case BTN_0: button = WACOMBUTTON_BT0; break;
+		case BTN_1: button = WACOMBUTTON_BT1; break;
+		case BTN_2: button = WACOMBUTTON_BT2; break;
+		case BTN_3: button = WACOMBUTTON_BT3; break;
+		case BTN_4: button = WACOMBUTTON_BT4; break;
+		case BTN_5: button = WACOMBUTTON_BT5; break;
+		case BTN_6: button = WACOMBUTTON_BT6; break;
+		case BTN_7: button = WACOMBUTTON_BT7; break;
 		case BTN_TOOL_PEN: tool = WACOMTOOLTYPE_PEN; break;
 		case BTN_TOOL_PENCIL: tool = WACOMTOOLTYPE_PENCIL; break;
 		case BTN_TOOL_BRUSH: tool = WACOMTOOLTYPE_BRUSH; break;
 		case BTN_TOOL_RUBBER: tool = WACOMTOOLTYPE_ERASER; break;
 		case BTN_TOOL_AIRBRUSH: tool = WACOMTOOLTYPE_AIRBRUSH; break;
 		case BTN_TOOL_MOUSE: tool = WACOMTOOLTYPE_MOUSE; break;
-		case BTN_TOOL_FINGER: break;
+		case BTN_TOOL_FINGER: tool = WACOMTOOLTYPE_PAD; break;
 		case BTN_TOOL_LENS: tool = WACOMTOOLTYPE_LENS; break;
 	}
 
@@ -802,11 +840,11 @@ static int USBParseData(WACOMTABLET_PRIV* pTablet,
 		case EV_SYN: /* kernel 2.6 */
 #endif
 		case EV_MSC: /* kernel 2.4 */
-			     if (USBParseMSC(pUSB,pEv)) return 1; break;
-		case EV_KEY: if (USBParseKEY(pUSB,pEv)) return 1; break;
-		case EV_ABS: if (USBParseABS(pUSB,pEv)) return 1; break;
-		case EV_REL: if (USBParseREL(pUSB,pEv)) return 1; break;
-		default: errno = EINVAL; return 1;
+			     if (USBParseMSC(pUSB,pEv)) return pEv->type; break;
+		case EV_KEY: if (USBParseKEY(pUSB,pEv)) return pEv->type; break;
+		case EV_ABS: if (USBParseABS(pUSB,pEv)) return pEv->type; break;
+		case EV_REL: if (USBParseREL(pUSB,pEv)) return pEv->type; break;
+		default: errno = EINVAL; return pEv->type;
 	}
 
 	return pState ? WacomCopyState(pState,&pUSB->state) : 0;
