@@ -1,5 +1,6 @@
 /*
- * Copyright 1995-2004 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 2002-2005 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
@@ -44,8 +45,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	const WacomChannelPtr pChannel);
 static void resetSampleCounter(const WacomChannelPtr pChannel);
 static void sendAButton(LocalDevicePtr local, int button, int mask,
-		int rx, int ry, int rz, int rtx, int rty, int rrot,
-		int rth, int rwheel);
+		int rx, int ry, int rz, int v3, int v4, int v5);
  
 /*****************************************************************************
  * xf86WcmSetScreen --
@@ -55,16 +55,24 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
  *   combined horizontal and vertical setups
  ****************************************************************************/
 
-static void xf86WcmSetScreen(LocalDevicePtr local, int *v0, int *v1)
+static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	int screenToSet = 0;
 	int totalWidth = 0, maxHeight = 0, leftPadding = 0;
-	int i, x, y;
+	int i, x, y, v0 = *value0, v1 = *value1;
 	double sizeX = priv->bottomX - priv->topX - 2*priv->tvoffsetX;
 	double sizeY = priv->bottomY - priv->topY - 2*priv->tvoffsetY;
 
-	DBG(6, ErrorF("xf86WcmSetScreen\n"));
+	DBG(6, ErrorF("xf86WcmSetScreen v0=%d v1=%d\n", *value0, *value1));
+
+	if (priv->twinview == TV_NONE && (priv->flags & ABSOLUTE_FLAG))
+	{
+		v0 = v0 > priv->bottomX ? priv->bottomX - priv->topX :
+			v0 < priv->topX ? 0 : v0 - priv->topX;
+		v1 = v1 > priv->bottomY ? priv->bottomY - priv->topY :
+			v1 < priv->topY ? 0 : v1 - priv->topY;
+	}
 
 	/* set factorX and factorY for single screen setup since
 	 * Top X Y and Bottom X Y can be changed while driver is running
@@ -77,16 +85,16 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *v0, int *v1)
 			{
 				if (priv->twinview == TV_LEFT_RIGHT)
 				{
-					if (*v0 > priv->bottomX - priv->tvoffsetX && *v0 <= priv->bottomX)
+					if (v0 > priv->bottomX - priv->tvoffsetX && v0 <= priv->bottomX)
 						priv->currentScreen = 1;
-					if (*v0 > priv->topX && *v0 <= priv->topX + priv->tvoffsetX)
+					if (v0 > priv->topX && v0 <= priv->topX + priv->tvoffsetX)
 						priv->currentScreen = 0;
 				}
 				if (priv->twinview == TV_ABOVE_BELOW)
 				{
-					if (*v1 > priv->bottomY - priv->tvoffsetY && *v1 <= priv->bottomY)
+					if (v1 > priv->bottomY - priv->tvoffsetY && v1 <= priv->bottomY)
 						priv->currentScreen = 1;
-					if (*v1 > priv->topY && *v1 <= priv->topY + priv->tvoffsetY)
+					if (v1 > priv->topY && v1 <= priv->topY + priv->tvoffsetY)
 						priv->currentScreen = 0;
 				}
 			}
@@ -129,7 +137,7 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *v0, int *v1)
 	{
 		for (i = 0; i < priv->numScreen; i++)
 		{
-			if (*v0 * totalWidth <= (leftPadding + 
+			if (v0 * totalWidth <= (leftPadding + 
 				screenInfo.screens[i]->width) * sizeX)
 			{
 				screenToSet = i;
@@ -144,10 +152,10 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *v0, int *v1)
 		screenToSet = priv->screen_no;
 		for (i = 0; i < screenToSet; i++)
 			leftPadding += screenInfo.screens[i]->width;
-		*v0 = (sizeX * leftPadding + *v0
+		v0 = (sizeX * leftPadding + v0
 			* screenInfo.screens[screenToSet]->width) /
 			(double)totalWidth + 0.5;
-		*v1 = *v1 * screenInfo.screens[screenToSet]->height /
+		v1 = v1 * screenInfo.screens[screenToSet]->height /
 			(double)maxHeight + 0.5;
 	}
 
@@ -155,9 +163,9 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *v0, int *v1)
 	{
 		priv->factorX = totalWidth/sizeX;
 		priv->factorY = maxHeight/sizeY;
-		x = (*v0 - sizeX
+		x = (v0 - sizeX
 			* leftPadding / totalWidth) * priv->factorX + 0.5;
-		y = *v1 * priv->factorY + 0.5;
+		y = v1 * priv->factorY + 0.5;
 		
 		if (x >= screenInfo.screens[screenToSet]->width)
 			x = screenInfo.screens[screenToSet]->width - 1;
@@ -168,15 +176,15 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *v0, int *v1)
 #endif
 	{
 		if (priv->screen_no == -1)
-			*v0 = (*v0 * totalWidth - sizeX * leftPadding)
+			v0 = (v0 * totalWidth - sizeX * leftPadding)
 				/ screenInfo.screens[screenToSet]->width;
 		else
 			screenToSet = priv->screen_no;
 		priv->factorX = screenInfo.screens[screenToSet]->width / sizeX;
 		priv->factorY = screenInfo.screens[screenToSet]->height / sizeY;
 
-		x = *v0 * priv->factorX + 0.5;
-		y = *v1 * priv->factorY + 0.5;
+		x = v0 * priv->factorX + 0.5;
+		y = v1 * priv->factorY + 0.5;
 	}
 
 	xf86XInputSetScreen(local, screenToSet, x, y);
@@ -191,9 +199,8 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *v0, int *v1)
  *   previous one.
  ****************************************************************************/
 
-static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
-		int rx, int ry, int rz, int rtx, int rty, int rrot,
-		int rth, int rwheel)
+static void xf86WcmSendButtons(LocalDevicePtr local, int buttons, int rx, int ry, 
+		int rz, int v3, int v4, int v5)
 {
 	int button, mask, bsent = 0;
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
@@ -201,7 +208,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
 	DBG(6, ErrorF("xf86WcmSendButtons buttons=%d for %s\n", buttons, local->name));
 
 	/* Tablet PC buttons. */
-	if ( common->wcmTPCButton && !IsCursor(priv) && !IsPad(priv) )
+	if ( common->wcmTPCButton && !IsCursor(priv) && !IsPad(priv) && !IsEraser(priv) )
 	{
 		if ( buttons & 1 )
 		{
@@ -220,7 +227,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
 						bsent = 1;
 						/* set to the configured button */
 						sendAButton(local, priv->button[button-1], 1, rx, ry, 
-							rz, rtx, rty, rrot, rth, rwheel);
+							rz, v3, v4, v5);
 					}
 				}
 				
@@ -233,7 +240,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
 				{
 					priv->flags |= TPCBUTTONONE_FLAG;
 					sendAButton(local, priv->button[0], 1, rx, ry, 
-						rz, rtx, rty, rrot, rth, rwheel);
+						rz, v3, v4, v5);
 				}
 			}
 			else
@@ -252,13 +259,13 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
 						if (priv->flags & TPCBUTTONONE_FLAG && !bsent)
 						{
 							priv->flags &= ~TPCBUTTONONE_FLAG;
-							sendAButton(local, 1, 0, rx, ry, rz, 
-								rtx, rty, rrot, rth, rwheel);
+							sendAButton(local, priv->button[0], 0, rx, ry, rz, 
+								v3, v4, v5);
 							bsent = 1;
 						}
 						/* set to the configured buttons */
 						sendAButton(local, priv->button[button-1], mask & buttons, 
-							rx, ry, rz, rtx, rty, rrot, rth, rwheel);
+							rx, ry, rz, v3, v4, v5);
 					}
 				}
 			}
@@ -275,7 +282,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
 				{
 					/* set to the configured button */
 					sendAButton(local, priv->button[button-1], 0, rx, ry, 
-						rz, rtx, rty, rrot, rth, rwheel);
+						rz, v3, v4, v5);
 				}
 			}
 			/* This is also part of the workaround of the XFree86 bug mentioned above
@@ -283,7 +290,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
 			if (priv->flags & TPCBUTTONONE_FLAG)
 			{
 				priv->flags &= ~TPCBUTTONONE_FLAG;
-				sendAButton(local, 1, 0, rx, ry, rz, rtx, rty, rrot, rth, rwheel);
+				sendAButton(local, priv->button[0], 0, rx, ry, rz, v3, v4, v5);
 			}
 		}
 	}
@@ -296,7 +303,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
 			{
 				/* set to the configured button */
 				sendAButton(local, priv->button[button-1], mask & buttons, rx, ry, 
-					rz, rtx, rty, rrot, rth, rwheel);
+					rz, v3, v4, v5);
 			}
 		}
 	}
@@ -308,8 +315,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons,
  ****************************************************************************/
 
 static void sendAButton(LocalDevicePtr local, int button, int mask,
-		int rx, int ry, int rz, int rtx, int rty, int rrot,
-		int rth, int rwheel)
+		int rx, int ry, int rz, int v3, int v4, int v5)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	WacomCommonPtr common = priv->common;
@@ -325,28 +331,17 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		if ( mask )
 		{
 			/* Left button down */
-			if (IsCursor(priv))
 				xf86PostButtonEvent(local->dev, is_absolute, nbutton, 
-					1, 0, 6, rx, ry, rz, rrot, rth, rwheel);
-			else
-				xf86PostButtonEvent(local->dev, is_absolute, nbutton,
-					1, 0, 6, rx, ry, rz, rtx, rty, rwheel);
+					1,0,6,rx,ry,rz,v3,v4,v5);
+
 			/* Left button up */
-			if (IsCursor(priv))
-				xf86PostButtonEvent(local->dev, is_absolute, nbutton, 
-					0, 0, 6, rx, ry, rz, rrot, rth, rwheel);
-			else
-				xf86PostButtonEvent(local->dev, is_absolute, nbutton, 
-					0, 0, 6, rx, ry, rz, rtx, rty, rwheel);
+			xf86PostButtonEvent(local->dev, is_absolute, nbutton, 
+					0,0,6,rx,ry,rz,v3,v4,v5);
 		}
 
 		/* Left button down/up upon mask is 1/0 */
-		if (IsCursor(priv))
-			xf86PostButtonEvent(local->dev, is_absolute, nbutton, 
-				mask != 0, 0, 6, rx, ry, rz, rrot, rth, rwheel);
-		else
-			xf86PostButtonEvent(local->dev, is_absolute, nbutton, 
-				mask != 0, 0, 6, rx, ry, rz, rtx, rty, rwheel);
+		xf86PostButtonEvent(local->dev, is_absolute, nbutton, 
+				mask != 0,0,6,rx,ry,rz,v3,v4,v5);
 	}
 	/* switch absolute or relative (Mode Toggle) */
 	if ( button == 19 && mask )
@@ -364,12 +359,8 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 	}
 	if (button < 17)
 	{
-		if (IsCursor(priv))
-			xf86PostButtonEvent(local->dev, is_absolute, button,
-				mask != 0, 0, 6, rx, ry, rz, rrot, rth, rwheel);
-		else
-			xf86PostButtonEvent(local->dev, is_absolute, button,
-				mask != 0, 0, 6, rx, ry, rz, rtx, rty, rwheel);
+		xf86PostButtonEvent(local->dev, is_absolute, button,
+				mask != 0,0,6,rx,ry,rz,v3,v4,v5);
 	}
 }
 
@@ -381,6 +372,8 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigned int channel)
 {
 	int type = ds->device_type;
+	int id = ds->device_id;
+	int serial = (int)ds->serial_num;
 	int is_button = !!(ds->buttons);
 	int is_proximity = ds->proximity;
 	int x = ds->x;
@@ -392,16 +385,23 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigne
 	int rot = ds->rotation;
 	int throttle = ds->throttle;
 	int wheel = ds->abswheel;
-
+	int tmp_coord;
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	WacomCommonPtr common = priv->common;
-	int rx, ry, rz, rtx, rty, rrot, rth, rw, no_jitter;
+
+	int rx, ry, rz, v3, v4, v5, no_jitter;
 	double param, relacc;
-	int is_core_pointer, is_absolute, doffsetX=0, doffsetY=0;
-	int aboveBelowSwitch = (priv->twinview == TV_ABOVE_BELOW)
-		? ((y < priv->topY) ? -1 : ((priv->bottomY < y) ? 1 : 0)) : 0;
-	int leftRightSwitch = (priv->twinview == TV_LEFT_RIGHT)
-		? ((x < priv->topX) ? -1 : ((priv->bottomX < x) ? 1 : 0)) : 0;
+	int is_core_pointer, is_absolute;
+
+	if (priv->serial && serial != priv->serial)
+	{
+		DBG(10, ErrorF("[%s] serial number is %d but your system configured %d", 
+			local->name, serial, (int)priv->serial));
+		return;
+	}
+
+	priv->currentX = x;
+	priv->currentY = y;
 
 	/* use tx and ty to report stripx and stripy */
 	if (type == PAD_ID)
@@ -419,80 +419,63 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigne
 		x, y, z, is_button ? "true" : "false", buttons,
 		tx, ty, wheel, rot, throttle));
 
-	is_absolute = (priv->flags & ABSOLUTE_FLAG);
-	is_core_pointer = xf86IsCorePointer(local->dev);
-
-	if ( is_proximity || x || y || z || buttons || tx || ty || wheel )
+	/* rotation mixes x and y up a bit */
+	if (common->wcmRotate == ROTATE_CW)
 	{
-		switch ( leftRightSwitch )
-		{
-			case -1:
-				doffsetX = 0;
-				break;
-			case 1:
-				doffsetX = priv->bottomX - priv->topX - 2*priv->tvoffsetX;
-				break;
-		}
-		switch ( aboveBelowSwitch )
-		{
-			case -1:
-				doffsetY = 0;
-				break;
-			case 1:
-				doffsetY = priv->bottomY - priv->topY - 2*priv->tvoffsetY;
-				break;
-		}
+		tmp_coord = x;
+		x = y;
+		y = common->wcmMaxY - tmp_coord;
+	}
+	else if (common->wcmRotate == ROTATE_CCW)
+	{
+		tmp_coord = y;
+		y = x;
+		x = common->wcmMaxX - tmp_coord;
 	}
 
-	x += doffsetX;
-	y += doffsetY;
-
-	DBG(6, ErrorF("[%s] %s prox=%d\tx=%d\ty=%d\tz=%d\t"
-		"button=%s\tbuttons=%d\t on channel=%d\n",
-		local->name,
-		is_absolute ? "abs" : "rel",
-		is_proximity,
-		x, y, z,
-		is_button ? "true" : "false", buttons, channel));
+	is_absolute = (priv->flags & ABSOLUTE_FLAG);
+	is_core_pointer = xf86IsCorePointer(local->dev);
 
 	/* sets rx and ry according to the mode */
 	if (is_absolute)
 	{
-		if (priv->twinview == TV_NONE)
+		rx = x;
+		ry = y;
+		rz = z;
+		if (IsCursor(priv)) 
 		{
-			rx = x > priv->bottomX ? priv->bottomX - priv->topX :
-				x < priv->topX ? 0 : x - priv->topX;
-			ry = y > priv->bottomY ? priv->bottomY - priv->topY :
-				y < priv->topY ? 0 : y - priv->topY;
+			v3 = rot;
+			v4 = throttle;
 		}
 		else
 		{
-			rx = x;
-			ry = y;
+			v3 = tx;
+			v4 = ty;
 		}
-		rz = z;
-		rtx = tx;
-		rty = ty;
-		rrot = rot;
-		rth = throttle;
-		rw = wheel;
+		v5 = wheel;
 	}
 	else
 	{
 		if (priv->oldProximity)
 		{
-			/* unify acceleration in both directions */
-			rx = (x - priv->oldX) * priv->factorY / priv->factorX;
+			if (x > priv->bottomX || x < priv->topX)
+				x = priv->oldX;
+			if (y > priv->bottomY || y < priv->topY)
+				y = priv->oldY;
+
+			rx = x - priv->oldX;
 			ry = y - priv->oldY;
+			rz = z - priv->oldZ;
 		}
 		else
 		{
 			rx = 0;
 			ry = 0;
+			rz = 0;
 		}
 
 		/* don't apply speed for fairly small increments */
-		no_jitter = priv->speed * 3;
+		no_jitter = (priv->speed*3 > 4) ? priv->speed*3 : 4;
 		relacc = (MAX_ACCEL-priv->accel)*(MAX_ACCEL-priv->accel);
 		if (ABS(rx) > no_jitter)
 		{
@@ -505,9 +488,10 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigne
 				/* don't apply acceleration when too fast. */
 				if (param < 20.00)
 				{
-					rx *= param;
+					param = 20.00;
 				}
 			}
+			rx *= param;
 		}
 		if (ABS(ry) > no_jitter)
 		{
@@ -522,60 +506,74 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigne
 					ry *= param;
 				}
 			}
+			ry *= param;
 		}
-		rz = z - priv->oldZ;
-		rtx = tx - priv->oldTiltX;
-		rty = ty - priv->oldTiltY;
-		rrot = rot - priv->oldRot;
-		rth = throttle - priv->oldThrottle;
-		rw = wheel - priv->oldWheel;
+		if (IsCursor(priv)) 
+		{
+			v3 = rot - priv->oldRot;
+			v4 = throttle - priv->oldThrottle;
+		}
+		else
+		{
+			v3 = tx - priv->oldTiltX;
+			v4 = ty - priv->oldTiltY;
+		}
+		v5 = wheel - priv->oldWheel;
 	}
 
-	/* for multiple monitor support, we need to set the proper 
-	 * screen and modify the axes before posting events */
-	if( !(priv->flags & BUTTONS_ONLY_FLAG) || !channel )
+	priv->currentX = rx;
+	priv->currentY = ry;
+
+	DBG(6, ErrorF("[%s] %s prox=%d\tx=%d\ty=%d\tz=%d\tv3=%d\tv4=%d\tv5=%d\tid=%d"
+		"\tserial=%d\tbutton=%s\tbuttons=%d\t on channel=%d\n",
+		local->name,
+		is_absolute ? "abs" : "rel",
+		is_proximity,
+		rx, ry, rz, v3, v4, v5, id, serial,
+		is_button ? "true" : "false", buttons, channel));
+
+	/* report tool id in the upper 2 bytes of 4th valuator 
+	 * serial number in the upper 2 bytes of 5th and 6th 
+	 * valuators for the 4 bytes of serial number .
+	 * All tools except pad
+	 */
+	if (type != PAD_ID)
 	{
-		xf86WcmSetScreen(local, &rx, &ry);
+		v3 = ((id<<16) & 0xffff0000) | (((short)v3)& 0xffff);
+		v4 = (serial & 0xffff0000) | (((short)v4)& 0xffff);
+		v5 = ((serial<<16) & 0xffff0000) | (((short)v5)& 0xffff);
 	}
 
 	/* coordinates are ready we can send events */
 	if (is_proximity)
 	{
-		if (!priv->oldProximity)
+		/* for multiple monitor support, we need to set the proper 
+		 * screen and modify the axes before posting events */
+		if( !(priv->flags & BUTTONS_ONLY_FLAG) )
 		{
-			if (IsCursor(priv))
-				xf86PostProximityEvent(
-					local->dev, 1, 0, 6,
-					rx, ry, rz, rrot,
-					rth, rw);
-			else
-				xf86PostProximityEvent(
-					local->dev, 1, 0, 6,
-					rx, ry, rz, rtx, rty,
-					rw);
+			xf86WcmSetScreen(local, &rx, &ry);
 		}
 
-		/* don't move the cursor if it only supports buttons or
-		 * if it's the second tool in dual input case.
-		 * More complicated dual input, such as only sylus/puck moves 
-		 * the cursor or both tools can move the cursor will be 
-		 * supported when needed */
-		if( !(priv->flags & BUTTONS_ONLY_FLAG) || !channel )
+		/* unify acceleration in both directions for relative mode to draw a circle */
+		if (!is_absolute)
+			rx *= priv->factorY / priv->factorX;
+
+		if (!priv->oldProximity)
 		{
-			if (IsCursor(priv))
-				xf86PostMotionEvent(local->dev,
-					is_absolute, 0, 6, rx, ry, rz,
-					rrot, rth, rw);
-			else
-				xf86PostMotionEvent(local->dev,
-					is_absolute, 0, 6, rx, ry, rz,
-					rtx, rty, rw);
+				xf86PostProximityEvent(local->dev, 1, 0, 6,
+					rx, ry, rz, v3, v4, v5);
+		}
+
+		/* don't move the cursor if it only supports buttons */
+		if( !(priv->flags & BUTTONS_ONLY_FLAG) )
+		{
+				xf86PostMotionEvent(local->dev, is_absolute, 
+					0, 6, rx, ry, rz, v3, v4, v5);
 		}
 
 		if (priv->oldButtons != buttons)
 		{
-			xf86WcmSendButtons (local, buttons, rx, ry, rz,
-					rtx, rty, rrot, rth, rw);
+			xf86WcmSendButtons(local,buttons,rx,ry,rz,v3,v4,v5);
 		}
 
 		/* simulate button 4 and 5 for relative wheel */
@@ -585,14 +583,12 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigne
 			int i;
 			for (i=0; i<abs(ds->relwheel); i++)
 			{
-				xf86PostButtonEvent(local->dev,
-					is_absolute,
+				xf86PostButtonEvent(local->dev, is_absolute,
 					fakeButton, 1, 0, 6, rx, ry, rz,
-					rrot, rth, rw);
-				xf86PostButtonEvent(local->dev,
-					is_absolute,
+					v3, v4, v5);
+				xf86PostButtonEvent(local->dev, is_absolute,
 					fakeButton, 0, 0, 6, rx, ry, rz,
-					rrot, rth, rw);
+					v3, v4, v5);
 			}
 		}
 	}
@@ -605,8 +601,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigne
 		if (priv->oldButtons)
 		{
 			buttons = 0;
-			xf86WcmSendButtons (local, buttons, rx, ry, rz,
-				rtx, rty, rrot, rth, rw);
+			xf86WcmSendButtons(local,buttons,rx,ry,rz,v3,v4,v5);
 		}
 		if (!is_core_pointer)
 		{
@@ -615,49 +610,24 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigne
 			{
 				int macro = z / 2;
 
-				DBG(6, ErrorF("macro=%d buttons=%d "
-					"wacom_map[%d]=%lx\n",
-					macro, buttons, macro,
-					gWacomModule.keymap[macro]));
+				DBG(6, ErrorF("macro=%d buttons=%d \n",
+					macro, buttons));
 
 				/* First available Keycode begins at 8
 				 * therefore macro+7 */
 
 				/* key down */
-				if (IsCursor(priv))
-					xf86PostKeyEvent(local->dev,macro+7,1,
-						is_absolute,0,6,
-						0,0,buttons,rrot,rth,
-						rw);
-				else
-					xf86PostKeyEvent(local->dev,macro+7,1,
-						is_absolute,0,6,
-						0,0,buttons,rtx,rty,rw);
+				xf86PostKeyEvent(local->dev,macro+7,1,is_absolute,
+						0,6,0,0,buttons,v3,v4,v5);
 
 				/* key up */
-				if (IsCursor(priv))
-					xf86PostKeyEvent(local->dev,macro+7,0,
-						is_absolute,0,6,
-						0,0,buttons,rrot,rth,
-						rw);
-				else
-					xf86PostKeyEvent(local->dev,macro+7,0,
-						is_absolute,0,6,
-						0,0,buttons,rtx,rty,rw);
+				xf86PostKeyEvent(local->dev,macro+7,0,is_absolute,
+						0,6,0,0,buttons,v3,v4,v5);
 
 			}
 		}
 		if (priv->oldProximity)
-		{
-			if (IsCursor(priv))
-				xf86PostProximityEvent(local->dev,
-						0, 0, 6, rx, ry, rz,
-						rrot, rth, rw);
-			else
-				xf86PostProximityEvent(local->dev,
-						0, 0, 6, rx, ry, rz,
-						rtx, rty, rw);
-		}
+			xf86PostProximityEvent(local->dev,0,0,6,rx,ry,rz,v3,v4,v5);
 	} /* not in proximity */
 
 	priv->oldProximity = is_proximity;
@@ -684,6 +654,7 @@ static int xf86WcmSuppress(int suppress, const WacomDeviceState* dsOrig,
 	const WacomDeviceState* dsNew)
 {
 	/* NOTE: Suppression value of zero disables suppression. */
+	DBG(11, ErrorF("xf86WcmSuppress checking data (suppress=%d)\n", suppress));
 
 	if (dsOrig->buttons != dsNew->buttons) return 0;
 	if (dsOrig->proximity != dsNew->proximity) return 0;
@@ -705,6 +676,7 @@ static int xf86WcmSuppress(int suppress, const WacomDeviceState* dsOrig,
 	if ((ABS(dsOrig->abswheel - dsNew->abswheel) > suppress) ||
 		(dsNew->relwheel != 0)) return 0;
 
+	DBG(11, ErrorF("xf86WcmSuppress discarded data\n"));
 	return 1;
 }
 
@@ -766,6 +738,8 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 	WacomDeviceState* pLast;
 	WacomDeviceState ds;
 	WacomChannelPtr pChannel;
+	WacomFilterState fs;
+	int i;
 
 	/* tool on the tablet when driver starts */
 	if (!miPointerCurrentScreen())
@@ -789,7 +763,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 	/* timestamp the state for velocity and acceleration analysis */
 	ds.sample = (int)GetTimeInMillis();
 
-	DBG(10, ErrorF("xf86WcmEvent: c=%d i=%d t=%d s=%x x=%d y=%d b=%d "
+	DBG(10, ErrorF("xf86WcmEvent: c=%d i=%d t=%d s=%u x=%d y=%d b=%d "
 		"p=%d rz=%d tx=%d ty=%d aw=%d rw=%d t=%d df=%d px=%d st=%d\n",
 		channel,
 		ds.device_id,
@@ -800,49 +774,73 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		ds.tilty, ds.abswheel, ds.relwheel, ds.throttle,
 		ds.discard_first, ds.proximity, ds.sample));
 
-	DBG(11, ErrorF("filter %d, %p\n",RAW_FILTERING(common),
-		(void *)common->wcmModel->FilterRaw));
-
+#ifdef LINUX_INPUT
 	/* Discard the first 2 USB packages due to events delay */
-	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) )
+	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && ds.device_type != PAD_ID )
 	{
 		DBG(11, ErrorF("discarded %dth USB data.\n", pChannel->nSamples));
-		/* store channel device state for later use */
-		memmove(pChannel->valid.states + 1,
-			pChannel->valid.states,
-			sizeof(WacomDeviceState) * (MAX_SAMPLES - 1));
 		++pChannel->nSamples;
 		return; /* discard */
 	}
-
-	/* Filter raw data, fix hardware defects, perform error correction */
-	if (RAW_FILTERING(common) && common->wcmModel->FilterRaw)
+#endif
+	fs = pChannel->rawFilter;
+	if (!fs.npoints && ds.proximity)
 	{
-		if (common->wcmModel->FilterRaw(common,pChannel,&ds))
+		DBG(11, ErrorF("initialize Channel data.\n"));
+		/* store channel device state for later use */
+		for (i=MAX_SAMPLES - 1; i>=0; i--)
 		{
-			DBG(10, ErrorF("Raw filtering discarded data.\n"));
-			resetSampleCounter(pChannel);
-			return; /* discard */
+			fs.x[i]= ds.x;
+			fs.y[i]= ds.y;
 		}
-	}
-
-	/* Discard unwanted data */
-	if (xf86WcmSuppress(common->wcmSuppress, pLast, &ds) && pChannel->rawFilter.npoints == 4)
-	{
-		/* If throttle is not in use, discard data. */
-		if (ABS(ds.throttle) < common->wcmSuppress)
+		++fs.npoints;
+	} else  {
+		/* Filter raw data, fix hardware defects, perform error correction */
+		for (i=MAX_SAMPLES - 1; i>0; i--)
 		{
-			resetSampleCounter(pChannel);
-			return;
+			fs.x[i]= fs.x[i-1];
+			fs.y[i]= fs.y[i-1];
+		}
+		fs.x[0] = ds.x;
+		fs.y[0] = ds.y;
+		if (HANDLE_TILT(common) && (ds.device_type == STYLUS_ID || ds.device_type == ERASER_ID))
+		{
+			for (i=MAX_SAMPLES - 1; i>0; i--)
+			{
+				fs.tiltx[i]= fs.tiltx[i-1];
+				fs.tilty[i]= fs.tilty[i-1];
+			}
+			fs.tiltx[0] = ds.tiltx;
+			fs.tilty[0] = ds.tilty;
+		}
+		if (RAW_FILTERING(common) && common->wcmModel->FilterRaw)
+		{
+			if (common->wcmModel->FilterRaw(common,pChannel,&ds))
+			{
+				DBG(10, ErrorF("Raw filtering discarded data.\n"));
+				resetSampleCounter(pChannel);
+				return; /* discard */
+			}
 		}
 
-		/* Otherwise, we need this event for time-rate-of-change
-		 * values like the throttle-to-relative-wheel filter.
-		 * To eliminate position change events, we reset all values
-		 * to last unsuppressed position. */
+		/* Discard unwanted data */
+		if (xf86WcmSuppress(common->wcmSuppress, pLast, &ds))
+		{
+			/* If throttle is not in use, discard data. */
+			if (ABS(ds.throttle) < common->wcmSuppress)
+			{
+				resetSampleCounter(pChannel);
+				return;
+			}
 
-		ds = *pLast;
-		RESET_RELATIVE(ds);
+			/* Otherwise, we need this event for time-rate-of-change
+			 * values like the throttle-to-relative-wheel filter.
+			 * To eliminate position change events, we reset all values
+		 	* to last unsuppressed position. */
+
+			ds = *pLast;
+			RESET_RELATIVE(ds);
+		}
 	}
 
 	/* JEJ - Do not move this code without discussing it with me.
@@ -856,7 +854,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		pChannel->valid.states,
 		sizeof(WacomDeviceState) * (MAX_SAMPLES - 1));
 	pChannel->valid.state = ds; /*save last raw sample */
-	if (pChannel->rawFilter.npoints < 4) ++pChannel->nSamples;
+	if (pChannel->nSamples < 4) ++pChannel->nSamples;
 
 	commonDispatchDevice(common,channel,pChannel);
 	resetSampleCounter(pChannel);
@@ -868,9 +866,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	int id, idx;
 	WacomDevicePtr priv;
 	LocalDevicePtr pDev = NULL;
-	LocalDevicePtr pLastDev = pChannel->pDev;
 	WacomDeviceState* ds = &pChannel->valid.states[0];
-	WacomDeviceState* pLast = &pChannel->valid.states[1];
 
 	DBG(10, ErrorF("commonDispatchEvents\n"));
 
@@ -900,33 +896,14 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 		if (id == ds->device_type &&
 			((!priv->serial) || (ds->serial_num == priv->serial)))
 		{
-			if ((priv->topX <= ds->x && priv->bottomX > ds->x &&
-				priv->topY <= ds->y && priv->bottomY > ds->y))
-			{
-				DBG(11, ErrorF("tool id=%d for %s\n",
+			DBG(11, ErrorF("tool id=%d for %s\n",
 					id, common->wcmDevices[idx]->name));
-				pDev = common->wcmDevices[idx];
-				break;
-			}
-			/* Fallback to allow the cursor to move
-			 * smoothly along screen edges */
-			else if (priv->oldProximity)
-			{
-				pDev = common->wcmDevices[idx];
-			}
+			pDev = common->wcmDevices[idx];
+			break;
 		}
 	}
 
-	DBG(11, ErrorF("commonDispatchEvents: %p %p\n",(void *)pDev,(void *)pLastDev));
-
-	/* if the logical device of the same physical tool has changed,
-	 * send proximity out to the previous one */
-	if (pLastDev && (pLastDev != pDev) &&
-		(pLast->serial_num == ds->serial_num))
-	{
-		pLast->proximity = 0;
-		xf86WcmSendEvents(pLastDev, pLast, channel);
-	}
+	DBG(11, ErrorF("commonDispatchEvents: %p \n",(void *)pDev));
 
 	/* if a device matched criteria, handle filtering per device
 	 * settings, and send event to XInput */
@@ -939,7 +916,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 		/* button 1 Threshold test */
 		int button = 1;
-		if ( !IsCursor(priv) )
+		if ( IsStylus(priv) || IsEraser(priv))
 		{
 			if (filtered.pressure < common->wcmThreshold )
 				filtered.buttons &= ~button;
@@ -996,14 +973,20 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 		#endif /* throttle */
 
-		/* force out-prox when height is greater than 112. 
-		 * This only applies to USB protocol V tablets
+		/* force out-prox when height is greater than 13 
+		 * (use 112 for history reason for now) for GD & XD;
+		 * 28 for PTZ. This only applies to USB protocol V tablets
 		 * which aimed at improving relative movement support. 
 		 */
-		if (filtered.distance > 112 && !(priv->flags & ABSOLUTE_FLAG) && !channel )
+		if (!(priv->flags & ABSOLUTE_FLAG) && IsCursor(priv))
 		{
-			ds->proximity = 0;
-			filtered.proximity = 0;
+			DBG(11, ErrorF("Distance over the tablet: %d \n", filtered.distance));
+			if ((filtered.distance > 28 && strstr(common->wcmModel->name, "Intuos3")) 
+			|| (filtered.distance > 112 && !strstr(common->wcmModel->name, "Intuos3")) )
+			{
+				ds->proximity = 0;
+				filtered.proximity = 0;
+			}
 		}
 
 		xf86WcmSendEvents(pDev, &filtered, channel);
@@ -1024,20 +1007,31 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
  * xf86WcmInitTablet -- common initialization for all tablets
  ****************************************************************************/
 
-int xf86WcmInitTablet(WacomCommonPtr common, WacomModelPtr model,
-	int fd, const char* id, float version)
+int xf86WcmInitTablet(LocalDevicePtr local, WacomModelPtr model,
+	const char* id, float version)
 {
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+	int temp;
+
 	/* Initialize the tablet */
-	model->Initialize(common,fd,id,version);
+	model->Initialize(common,id,version);
 
 	/* Get tablet resolution */
 	if (model->GetResolution)
-		model->GetResolution(common,fd);
+		model->GetResolution(local);
 
 	/* Get tablet range */
-	if (model->GetRanges && (model->GetRanges(common,fd) != Success))
+	if (model->GetRanges && (model->GetRanges(local) != Success))
 		return !Success;
 	
+	/* Rotation rotates the Max Y and Y */
+	if (common->wcmRotate==ROTATE_CW || common->wcmRotate==ROTATE_CCW)
+	{
+		temp = common->wcmMaxX;
+		common->wcmMaxX = common->wcmMaxY;
+		common->wcmMaxY = temp;
+	}
+
 	/* Default threshold value if not set */
 	if (common->wcmThreshold <= 0)
 	{
@@ -1048,7 +1042,7 @@ int xf86WcmInitTablet(WacomCommonPtr common, WacomModelPtr model,
 	}
 
 	/* Reset tablet to known state */
-	if (model->Reset && (model->Reset(common,fd) != Success))
+	if (model->Reset && (model->Reset(local) != Success))
 	{
 		ErrorF("Wacom xf86WcmWrite error : %s\n", strerror(errno));
 		return !Success;
@@ -1057,14 +1051,14 @@ int xf86WcmInitTablet(WacomCommonPtr common, WacomModelPtr model,
 	/* Enable tilt mode, if requested and available */
 	if ((common->wcmFlags & TILT_REQUEST_FLAG) && model->EnableTilt)
 	{
-		if (model->EnableTilt(common,fd) != Success)
+		if (model->EnableTilt(local) != Success)
 			return !Success;
 	}
 
 	/* Enable hardware suppress, if requested and available */
 	if ((common->wcmSuppress != 0) && model->EnableSuppress)
 	{
-		if (model->EnableSuppress(common,fd) != Success)
+		if (model->EnableSuppress(local) != Success)
 			return !Success;
 	}
 
@@ -1073,7 +1067,7 @@ int xf86WcmInitTablet(WacomCommonPtr common, WacomModelPtr model,
 	{
 		if (model->SetLinkSpeed)
 		{
-			if (model->SetLinkSpeed(common,fd) != Success)
+			if (model->SetLinkSpeed(local) != Success)
 				return !Success;
 		}
 		else
@@ -1095,7 +1089,7 @@ int xf86WcmInitTablet(WacomCommonPtr common, WacomModelPtr model,
 			HANDLE_TILT(common) ? "enabled" : "disabled");
   
 	/* start the tablet data */
-	if (model->Start && (model->Start(common,fd) != Success))
+	if (model->Start && (model->Start(local) != Success))
 		return !Success;
 
 	/*set the model */
