@@ -1,5 +1,6 @@
 /*
- * Copyright 1995-2004 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 2002-2005 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *                                                                            
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
@@ -29,41 +30,30 @@
 static Bool serialDetect(LocalDevicePtr pDev);
 static Bool serialInit(LocalDevicePtr pDev);
 
-static int serialInitTablet(WacomCommonPtr common, int fd);
-static void serialInitIntuos(WacomCommonPtr common, int fd,
-	const char* id, float version);
-static void serialInitIntuos2(WacomCommonPtr common, int fd,
-	const char* id, float version);
-static void serialInitCintiq(WacomCommonPtr common, int fd,
-	const char* id, float version);
-static void serialInitPenPartner(WacomCommonPtr common, int fd,
-	const char* id, float version);
-static void serialInitGraphire(WacomCommonPtr common, int fd,
-	const char* id, float version);
-static void serialInitProtocol4(WacomCommonPtr common, int fd,
-	const char* id, float version);
-static void serialGetResolution(WacomCommonPtr common, int fd);
-static int serialGetRanges(WacomCommonPtr common, int fd);
-static int serialResetIntuos(WacomCommonPtr common, int fd);
-static int serialResetCintiq(WacomCommonPtr common, int fd);
-static int serialResetPenPartner(WacomCommonPtr common, int fd);
-static int serialResetProtocol4(WacomCommonPtr common, int fd);
-static int serialEnableTiltProtocol4(WacomCommonPtr common, int fd);
-static int serialEnableSuppressProtocol4(WacomCommonPtr common, int fd);
-static int serialSetLinkSpeedIntuos(WacomCommonPtr common, int fd);
-static int serialSetLinkSpeedProtocol5(WacomCommonPtr common, int fd);
-static int serialStartTablet(WacomCommonPtr common, int fd);
-static int serialParseCintiq(WacomCommonPtr common,
-	const unsigned char* data);
-static int serialParseGraphire(WacomCommonPtr common,
-	const unsigned char* data);
-static int serialParseProtocol4(WacomCommonPtr common,
-	const unsigned char* data);
-static int serialParseProtocol5(WacomCommonPtr common,
-	const unsigned char* data);
-static void serialParseP4Common(WacomCommonPtr common,
-	const unsigned char* data, WacomDeviceState* last,
-	WacomDeviceState* ds);
+static int serialInitTablet(LocalDevicePtr local);
+static void serialInitIntuos(WacomCommonPtr common, const char* id, float version);
+static void serialInitIntuos2(WacomCommonPtr common, const char* id, float version);
+static void serialInitCintiq(WacomCommonPtr common, const char* id, float version);
+static void serialInitPenPartner(WacomCommonPtr common, const char* id, float version);
+static void serialInitGraphire(WacomCommonPtr common, const char* id, float version);
+static void serialInitProtocol4(WacomCommonPtr common, const char* id, float version);
+static void serialGetResolution(LocalDevicePtr local);
+static int serialGetRanges(LocalDevicePtr local);
+static int serialResetIntuos(LocalDevicePtr local);
+static int serialResetCintiq(LocalDevicePtr local);
+static int serialResetPenPartner(LocalDevicePtr local);
+static int serialResetProtocol4(LocalDevicePtr local);
+static int serialEnableTiltProtocol4(LocalDevicePtr local);
+static int serialEnableSuppressProtocol4(LocalDevicePtr local);
+static int serialSetLinkSpeedIntuos(LocalDevicePtr local);
+static int serialSetLinkSpeedProtocol5(LocalDevicePtr local);
+static int serialStartTablet(LocalDevicePtr local);
+static int serialParseCintiq(WacomCommonPtr common, const unsigned char* data);
+static int serialParseGraphire(WacomCommonPtr common, const unsigned char* data);
+static int serialParseProtocol4(WacomCommonPtr common, const unsigned char* data);
+static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data);
+static void serialParseP4Common(WacomCommonPtr common, const unsigned char* data, 
+	WacomDeviceState* last, WacomDeviceState* ds);
 
 /*****************************************************************************
  * Global Structures
@@ -420,7 +410,6 @@ static Bool serialDetect(LocalDevicePtr pDev)
 static Bool serialInit(LocalDevicePtr local)
 {
 	int err;
-	WacomCommonPtr common = ((WacomDevicePtr)(local->private))->common;
     
 	DBG(1, ErrorF("initializing serial tablet\n"));    
 
@@ -513,7 +502,7 @@ static Bool serialInit(LocalDevicePtr local)
 
 	xf86WcmFlushTablet(local->fd);
 
-	if (serialInitTablet(common,local->fd) == !Success)
+	if (serialInitTablet(local) == !Success)
 	{
 		SYSCALL(xf86WcmClose(local->fd));
 		local->fd = -1;
@@ -529,7 +518,7 @@ static Bool serialInit(LocalDevicePtr local)
  ****************************************************************************/
 
 
-static int serialInitTablet(WacomCommonPtr common, int fd)
+static int serialInitTablet(LocalDevicePtr local)
 {
 	int loop, idx;
 	char id[BUFFER_SIZE];
@@ -548,7 +537,7 @@ static int serialInitTablet(WacomCommonPtr common, int fd)
 	else
 	{
 		DBG(2, ErrorF("reading model\n"));
-		if (!xf86WcmSendRequest(fd, WC_MODEL, id, sizeof(id)))
+		if (!xf86WcmSendRequest(local->fd, WC_MODEL, id, sizeof(id)))
 			return !Success;
 
 		DBG(2, ErrorF("%s\n", id));
@@ -573,7 +562,8 @@ static int serialInitTablet(WacomCommonPtr common, int fd)
 			model = &serialIntuos;
 		else if (id[2] == 'X' && id[3] == 'D')
 			model = &serialIntuos2;
-		else if (id[2] == 'P' && id[3] == 'L')
+		else if ( (id[2] == 'P' && id[3] == 'L') ||
+			(id[2] == 'D' && id[3] == 'T') )
 			model = &serialCintiq;
 		else if (id[2] == 'C' && id[3] == 'T')
 			model = &serialPenPartner;
@@ -584,11 +574,10 @@ static int serialInitTablet(WacomCommonPtr common, int fd)
 
 	}
 
-	return xf86WcmInitTablet(common,model,fd,id,version);
+	return xf86WcmInitTablet(local,model,id,version);
 }
 
-static int serialParseGraphire(WacomCommonPtr common,
-	const unsigned char* data)
+static int serialParseGraphire(WacomCommonPtr common, const unsigned char* data)
 {
 	int n;
 	WacomDeviceState* last = &common->wcmChannel[0].valid.state;
@@ -604,12 +593,12 @@ static int serialParseGraphire(WacomCommonPtr common,
 	/* get pressure */
 	ds->pressure = ((data[6]&ZAXIS_BITS) << 2 ) +
 		((data[3]&ZAXIS_BIT) >> 1) +
-		((data[3]&PROXIMITY_BIT) >> 6) +
+		((data[0]&ZAXIS_BIT) >> 6) +
 		((data[6]&ZAXIS_SIGN_BIT) ? 0 : 0x100);
 
 	/* get buttons */
-	ds->buttons = (data[3] & 0x38) >> 3;
-
+	ds->buttons = (data[3] & BUTTONS_BITS) >> 3;
+	
 	/* requires button info, so it goes down here. */
 	serialParseP4Common(common,data,last,ds);
 
@@ -619,15 +608,13 @@ static int serialParseGraphire(WacomCommonPtr common,
 		ds->relwheel = (data[6] & 0x30) >> 4;
 		if (data[6] & 0x40)
 			ds->relwheel = -ds->relwheel;
-
 	}
 
 	xf86WcmEvent(common,0,ds);
 	return common->wcmPktLength;
 }
 
-static int serialParseCintiq(WacomCommonPtr common,
-	const unsigned char* data)
+static int serialParseCintiq(WacomCommonPtr common, const unsigned char* data)
 {
 	int n;
 	WacomDeviceState* last = &common->wcmChannel[0].valid.state;
@@ -650,15 +637,15 @@ static int serialParseCintiq(WacomCommonPtr common,
 	else
 	{
 		/* which tablets use this? */
-		/* PL550, PL800, and Graphire apparently */
+		/* PL550 and PL800 apparently */
 		ds->pressure = ((data[6]&ZAXIS_BITS) << 2 ) +
 			((data[3]&ZAXIS_BIT) >> 1) +
-			((data[3]&PROXIMITY_BIT) >> 6) +
+			((data[0]&ZAXIS_BIT) >> 6) +
 			((data[6]&ZAXIS_SIGN_BIT) ? 0 : 0x100);
 	}
 
 	/* get buttons */
-	ds->buttons = (data[3] & 0x38) >> 3;
+	ds->buttons = (data[3] & BUTTONS_BITS) >> 3;
 
 	/* requires button info, so it goes down here. */
 	serialParseP4Common(common,data,last,ds);
@@ -667,8 +654,7 @@ static int serialParseCintiq(WacomCommonPtr common,
 	return common->wcmPktLength;
 }
 
-static int serialParseProtocol4(WacomCommonPtr common,
-	const unsigned char* data)
+static int serialParseProtocol4(WacomCommonPtr common, const unsigned char* data)
 {
 	int n;
 	WacomDeviceState* last = &common->wcmChannel[0].valid.state;
@@ -701,8 +687,7 @@ static int serialParseProtocol4(WacomCommonPtr common,
 	return common->wcmPktLength;
 }
 
-static int serialParseProtocol5(WacomCommonPtr common,
-	const unsigned char* data)
+static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data)
 {
 	int n;
 	int have_data=0;
@@ -877,8 +862,7 @@ static int serialParseProtocol5(WacomCommonPtr common,
  * Model-specific functions
  ****************************************************************************/
 
-static void serialInitIntuos(WacomCommonPtr common, int fd,
-	const char* id, float version)
+static void serialInitIntuos(WacomCommonPtr common, const char* id, float version)
 {
 	DBG(2, ErrorF("detected an Intuos model\n"));
 
@@ -892,8 +876,7 @@ static void serialInitIntuos(WacomCommonPtr common, int fd,
 	common->wcmFlags |= TILT_ENABLED_FLAG;
 }
 
-static void serialInitIntuos2(WacomCommonPtr common, int fd,
-	const char* id, float version)
+static void serialInitIntuos2(WacomCommonPtr common, const char* id, float version)
 {
 	DBG(2, ErrorF("detected an Intuos2 model\n"));
 
@@ -907,14 +890,15 @@ static void serialInitIntuos2(WacomCommonPtr common, int fd,
 	common->wcmFlags |= TILT_ENABLED_FLAG;
 }
 
-static void serialInitCintiq(WacomCommonPtr common, int fd,
-	const char* id, float version)
+static void serialInitCintiq(WacomCommonPtr common, const char* id, float version)
 {
 	DBG(2, ErrorF("detected a Cintiq model\n"));
 
 	common->wcmProtocolLevel = 4;
 	common->wcmPktLength = 7;
 	common->wcmVersion = version;
+	common->wcmResolX = 508; /* tablet X resolution in points/inch */
+	common->wcmResolY = 508; /* tablet Y resolution in points/inch */
 
 	if (id[5] == '2')
 	{
@@ -965,18 +949,21 @@ static void serialInitCintiq(WacomCommonPtr common, int fd,
 			common->wcmMaxZ = 255;
 		}
 	}
+	else if (id[5] == '7')
+	{
+		/* PL-710  */
+		common->wcmMaxZ = 511;
+		common->wcmResolX = 2540;
+		common->wcmResolY = 2540;
+	}
 	else if (id[5] == '8')
 	{
 		/* PL-800  */
 		common->wcmMaxZ = 511;
 	}
-
-	common->wcmResolX = 508; /* tablet X resolution in points/inch */
-	common->wcmResolY = 508; /* tablet Y resolution in points/inch */
 }
 
-static void serialInitPenPartner(WacomCommonPtr common, int fd,
-	const char* id, float version)
+static void serialInitPenPartner(WacomCommonPtr common, const char* id, float version)
 {
 	DBG(2, ErrorF("detected a PenPartner model\n"));
 
@@ -984,13 +971,12 @@ static void serialInitPenPartner(WacomCommonPtr common, int fd,
 	common->wcmPktLength = 7;
 	common->wcmVersion = version;
 
-	common->wcmMaxZ = 256;
+	common->wcmMaxZ = 255;
 	common->wcmResolX = 1000; /* tablet X resolution in points/inch */
 	common->wcmResolY = 1000; /* tablet Y resolution in points/inch */
 }
 
-static void serialInitGraphire(WacomCommonPtr common, int fd,
-	const char* id, float version)
+static void serialInitGraphire(WacomCommonPtr common, const char* id, float version)
 {
 	DBG(2, ErrorF("detected a Graphire model\n"));
 
@@ -1001,13 +987,12 @@ static void serialInitGraphire(WacomCommonPtr common, int fd,
 	/* Graphire models don't answer WC_COORD requests */
 	common->wcmMaxX = 5103;
 	common->wcmMaxY = 3711;
-	common->wcmMaxZ = 512;
-	common->wcmResolX = 1000; /* tablet X resolution in points/inch */
-	common->wcmResolY = 1000; /* tablet Y resolution in points/inch */
+	common->wcmMaxZ = 511;
+	common->wcmResolX = 1016; /* tablet X resolution in points/inch */
+	common->wcmResolY = 1016; /* tablet Y resolution in points/inch */
 }
 
-static void serialInitProtocol4(WacomCommonPtr common, int fd,
-	const char* id, float version)
+static void serialInitProtocol4(WacomCommonPtr common, const char* id, float version)
 {
 	DBG(2, ErrorF("detected a Protocol4 model\n"));
 
@@ -1026,15 +1011,16 @@ static void serialInitProtocol4(WacomCommonPtr common, int fd,
 	}
 }
 
-static void serialGetResolution(WacomCommonPtr common, int fd)
+static void serialGetResolution(LocalDevicePtr local)
 {
 	int a, b;
 	char buffer[BUFFER_SIZE], header[BUFFER_SIZE];
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
 
 	if (!(common->wcmResolX && common->wcmResolY))
 	{
 		DBG(2, ErrorF("Requesting resolution from device\n"));
-		if (xf86WcmSendRequest(fd, WC_CONFIG, buffer,
+		if (xf86WcmSendRequest(local->fd, WC_CONFIG, buffer,
 			sizeof(buffer)))
 		{
 			DBG(2, ErrorF("%s\n", buffer));
@@ -1068,14 +1054,15 @@ static void serialGetResolution(WacomCommonPtr common, int fd)
 		common->wcmResolX, common->wcmResolY));
 }
 
-static int serialGetRanges(WacomCommonPtr common, int fd)
+static int serialGetRanges(LocalDevicePtr local)
 {
 	char buffer[BUFFER_SIZE];
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
 
 	if (!(common->wcmMaxX && common->wcmMaxY))
 	{
 		DBG(2, ErrorF("Requesting max coordinates\n"));
-		if (!xf86WcmSendRequest(fd, WC_COORD, buffer,
+		if (!xf86WcmSendRequest(local->fd, WC_COORD, buffer,
 			sizeof(buffer)))
 		{
 			ErrorF("WACOM: unable to read max coordinates. "
@@ -1100,69 +1087,70 @@ static int serialGetRanges(WacomCommonPtr common, int fd)
 	return Success;
 }
 
-static int serialResetIntuos(WacomCommonPtr common, int fd)
+static int serialResetIntuos(LocalDevicePtr local)
 {
 	int err;
-	SYSCALL(err = xf86WcmWrite(fd, intuos_setup_string,
+	SYSCALL(err = xf86WcmWrite(local->fd, intuos_setup_string,
 		strlen(intuos_setup_string)));
 	return (err == -1) ? !Success : Success;
 }
 
-static int serialResetCintiq(WacomCommonPtr common, int fd)
+static int serialResetCintiq(LocalDevicePtr local)
 {
 	int err;
 
-	SYSCALL(err = xf86WcmWrite(fd, WC_RESET, strlen(WC_RESET)));
+	SYSCALL(err = xf86WcmWrite(local->fd, WC_RESET, strlen(WC_RESET)));
   
 	if (xf86WcmWait(75)) return !Success;
 
-	SYSCALL(err = xf86WcmWrite(fd, pl_setup_string,
+	SYSCALL(err = xf86WcmWrite(local->fd, pl_setup_string,
 		strlen(pl_setup_string)));
 	if (err == -1) return !Success;
 
-	SYSCALL(err = xf86WcmWrite(fd, penpartner_setup_string,
+	SYSCALL(err = xf86WcmWrite(local->fd, penpartner_setup_string,
 		strlen(penpartner_setup_string)));
 
 	return (err == -1) ? !Success : Success;
 }
 
-static int serialResetPenPartner(WacomCommonPtr common, int fd)
+static int serialResetPenPartner(LocalDevicePtr local)
 {
 	int err;
-	SYSCALL(err = xf86WcmWrite(fd, penpartner_setup_string,
+	SYSCALL(err = xf86WcmWrite(local->fd, penpartner_setup_string,
 		strlen(penpartner_setup_string)));
 	return (err == -1) ? !Success : Success;
 }
 
-static int serialResetProtocol4(WacomCommonPtr common, int fd)
+static int serialResetProtocol4(LocalDevicePtr local)
 {
 	int err;
 
-	SYSCALL(err = xf86WcmWrite(fd, WC_RESET, strlen(WC_RESET)));
+	SYSCALL(err = xf86WcmWrite(local->fd, WC_RESET, strlen(WC_RESET)));
   
 	if (xf86WcmWait(75)) return !Success;
 
-	SYSCALL(err = xf86WcmWrite(fd, setup_string,
+	SYSCALL(err = xf86WcmWrite(local->fd, setup_string,
 		strlen(setup_string)));
 	if (err == -1) return !Success;
 
-	SYSCALL(err = xf86WcmWrite(fd, penpartner_setup_string,
+	SYSCALL(err = xf86WcmWrite(local->fd, penpartner_setup_string,
 		strlen(penpartner_setup_string)));
 	return (err == -1) ? !Success : Success;
 }
 
-static int serialEnableTiltProtocol4(WacomCommonPtr common, int fd)
+static int serialEnableTiltProtocol4(LocalDevicePtr local)
 {
 	return Success;
 }
 
-static int serialEnableSuppressProtocol4(WacomCommonPtr common, int fd)
+static int serialEnableSuppressProtocol4(LocalDevicePtr local)
 {
 	char buf[20];
 	int err;
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
 
 	sprintf(buf, "%s%d\r", WC_SUPPRESS, common->wcmSuppress);
-	SYSCALL(err = xf86WcmWrite(fd, buf, strlen(buf)));
+	SYSCALL(err = xf86WcmWrite(local->fd, buf, strlen(buf)));
 
 	if (err == -1)
 	{
@@ -1173,8 +1161,10 @@ static int serialEnableSuppressProtocol4(WacomCommonPtr common, int fd)
 	return Success;
 }
 
-static int serialSetLinkSpeedIntuos(WacomCommonPtr common, int fd)
+static int serialSetLinkSpeedIntuos(LocalDevicePtr local)
 {
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+
 	if ((common->wcmLinkSpeed == 38400) &&
 		(common->wcmVersion < 2.0F))
 	{
@@ -1183,13 +1173,14 @@ static int serialSetLinkSpeedIntuos(WacomCommonPtr common, int fd)
 		ErrorF("Switching to 19200\n");
 		common->wcmLinkSpeed = 19200;
 	}
-	return serialSetLinkSpeedProtocol5(common,fd);
+	return serialSetLinkSpeedProtocol5(local);
 }
 
-static int serialSetLinkSpeedProtocol5(WacomCommonPtr common, int fd)
+static int serialSetLinkSpeedProtocol5(LocalDevicePtr local)
 {
 	int err;
 	char* speed_init_string;
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
 
 	DBG(1, ErrorF("Switching serial link to %d\n",
 		common->wcmLinkSpeed));
@@ -1199,7 +1190,7 @@ static int serialSetLinkSpeedProtocol5(WacomCommonPtr common, int fd)
 		WC_V_38400 : WC_V_19200;
 
 	/* Switch the tablet to the requested speed */
-	SYSCALL(err = xf86WcmWrite(fd, speed_init_string,
+	SYSCALL(err = xf86WcmWrite(local->fd, speed_init_string,
 		strlen(speed_init_string)));
 
 	if (err == -1)
@@ -1213,18 +1204,18 @@ static int serialSetLinkSpeedProtocol5(WacomCommonPtr common, int fd)
 		return !Success;
 
 	/* Set speed of serial link to requested speed */
-	if (xf86WcmSetSerialSpeed(fd, common->wcmLinkSpeed) < 0)
+	if (xf86WcmSetSerialSpeed(local->fd, common->wcmLinkSpeed) < 0)
 		return !Success;
 
 	return Success;
 }
 
-static int serialStartTablet(WacomCommonPtr common, int fd)
+static int serialStartTablet(LocalDevicePtr local)
 {
 	int err;
 
 	/* Tell the tablet to start sending coordinates */
-	SYSCALL(err = xf86WcmWrite(fd, WC_START, strlen(WC_START)));
+	SYSCALL(err = xf86WcmWrite(local->fd, WC_START, strlen(WC_START)));
 
 	if (err == -1)
 	{
@@ -1244,6 +1235,9 @@ static void serialParseP4Common(WacomCommonPtr common,
 		((ds->buttons & 4) ? ERASER_ID : STYLUS_ID) :
 		CURSOR_ID;
 
+	/* for Graphire eraser */
+	if(ds->buttons & 8) cur_type = ERASER_ID;
+
 	/* proximity bit */
 	ds->proximity = (data[0] & PROXIMITY_BIT);
 
@@ -1251,12 +1245,22 @@ static void serialParseP4Common(WacomCommonPtr common,
 	ds->x = (((data[0] & 0x3) << 14) + (data[1] << 7) + data[2]);
 	ds->y = (((data[3] & 0x3) << 14) + (data[4] << 7) + data[5]);
 
-	/* first time into prox */
-	if (!last->proximity && ds->proximity) 
-		ds->device_type = cur_type;
+	/* handle tilt values only for stylus */
+	if (HANDLE_TILT(common) && is_stylus)
+	{
+		ds->tiltx = (data[7] & TILT_BITS);
+		ds->tilty = (data[8] & TILT_BITS);
+		if (data[7] & TILT_SIGN_BIT)
+			ds->tiltx -= 64;
+		if (data[8] & TILT_SIGN_BIT)
+			ds->tilty -= 64;
+	}
 
+	/* first time into prox */
+	if (!last->proximity && ds->proximity)
+		ds->device_type = cur_type;
 	/* check on previous proximity */
-	else if (is_stylus)
+	else if (is_stylus && ds->proximity)
 	{
 		/* we were fooled by tip and second
 		 * sideswitch when it came into prox */
@@ -1270,21 +1274,20 @@ static void serialParseP4Common(WacomCommonPtr common,
 		}
 	}
 
+	ds->device_id = (ds->device_type == CURSOR_ID) ? CURSOR_DEVICE_ID : STYLUS_DEVICE_ID;
+
+	/* don't send button event for eraser 
+	 * button 1 event will be sent by testing presure level
+	 */
+	if (ds->device_type == ERASER_ID) {
+		ds->buttons = 0;
+		ds->device_id = ERASER_DEVICE_ID;
+	}
+		
 	DBG(8, ErrorF("serialParseP4Common %s\n",
 		ds->device_type == CURSOR_ID ? "CURSOR" :
 		ds->device_type == ERASER_ID ? "ERASER " :
 		ds->device_type == STYLUS_ID ? "STYLUS" : "NONE"));
-
-	/* handle tilt values only for stylus */
-	if (HANDLE_TILT(common) && is_stylus)
-	{
-		ds->tiltx = (data[7] & TILT_BITS);
-		ds->tilty = (data[8] & TILT_BITS);
-		if (data[7] & TILT_SIGN_BIT)
-			ds->tiltx -= 64;
-		if (data[8] & TILT_SIGN_BIT)
-			ds->tilty -= 64;
-	}
 }
 
 /*****************************************************************************
