@@ -1,5 +1,6 @@
 /*
- * Copyright 1995-2004 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 2002-2005 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *                                                                            
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
@@ -28,11 +29,11 @@
 static Bool usbDetect(LocalDevicePtr);
 static Bool usbInit(LocalDevicePtr pDev);
 
-static void usbInitProtocol5(WacomCommonPtr common, int fd, const char* id,
+static void usbInitProtocol5(WacomCommonPtr common, const char* id,
 	float version);
-static void usbInitProtocol4(WacomCommonPtr common, int fd, const char* id,
+static void usbInitProtocol4(WacomCommonPtr common, const char* id,
 	float version);
-static int usbGetRanges(WacomCommonPtr common, int fd);
+static int usbGetRanges(LocalDevicePtr local);
 static int usbParse(WacomCommonPtr common, const unsigned char* data);
 static void usbParseEvent(WacomCommonPtr common,
 	const struct input_event* event);
@@ -120,6 +121,21 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial);
 		xf86WcmFilterCoord,   /* input filtering */
 	};
 
+	static WacomModel usbGraphire4 =
+	{
+		"USB Graphire4",
+		usbInitProtocol4,
+		NULL,                 /* resolution not queried */
+		usbGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		xf86WcmFilterCoord,   /* input filtering */
+	};
+
 	static WacomModel usbCintiq =
 	{
 		"USB Cintiq",
@@ -210,6 +226,35 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial);
 		xf86WcmFilterCoord,   /* input filtering */
 	};
 
+	static WacomModel usbVolito2 =
+	{
+		"USB Volito2",
+		usbInitProtocol4,
+		NULL,                 /* resolution not queried */
+		usbGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		xf86WcmFilterCoord,   /* input filtering */
+	};
+
+	static WacomModel usbCintiqV5 =
+	{
+		"USB Cintiq 21UX",
+		usbInitProtocol5,
+		NULL,                 /* resolution not queried */
+		usbGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		xf86WcmFilterIntuos,  /* input filtering recommended */
+	};
 
 /*****************************************************************************
  * usbDetect --
@@ -230,7 +275,18 @@ static Bool usbDetect(LocalDevicePtr local)
 /*		ErrorF("%s Wacom Kernel Input driver version is %d.%d.%d\n",
 				XCONFIG_PROBED, version >> 16,
 				(version >> 8) & 0xff, version & 0xff);
-*/		return 1;
+*/
+		/* Try to grab the event device so that data don't leak to /dev/input/mice */
+#ifdef EV_SYN
+		SYSCALL(err = ioctl(local->fd, EVIOCGRAB, (pointer)1));
+
+		if (err < 0) 
+			ErrorF("%s Wacom X driver can't grab event device, errno=%d\n",
+				local->name, errno);
+		else 
+			ErrorF("%s Wacom X driver grabbed event device\n", local->name);
+#endif
+		return 1;
 	}
 
 	return 0;
@@ -245,8 +301,7 @@ static Bool usbInit(LocalDevicePtr local)
 	short sID[4];
 	char id[BUFFER_SIZE];
 	WacomModelPtr model = NULL;
-	WacomDevicePtr priv = (WacomDevicePtr)local->private;
-	WacomCommonPtr common = priv->common;
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
 
 	DBG(1, ErrorF("initializing USB tablet\n"));    
 
@@ -257,6 +312,7 @@ static Bool usbInit(LocalDevicePtr local)
 	/* vendor is wacom */
 	if (sID[1] == 0x056A)
 	{
+		common->tablet_id = sID[2];
 		/* switch on product */
 		switch (sID[2])
 		{
@@ -274,6 +330,10 @@ static Bool usbInit(LocalDevicePtr local)
 			case 0x14: /* Graphire3 6x8 */
 				model = &usbGraphire3; break;
 
+			case 0x15: /* Graphire4 4x5 */
+			case 0x16: /* Graphire4 6x8 */
+				model = &usbGraphire4; break;
+
 			case 0x20: /* Intuos 4x5 */
 			case 0x21: /* Intuos 6x8 */
 			case 0x22: /* Intuos 9x12 */
@@ -284,12 +344,17 @@ static Bool usbInit(LocalDevicePtr local)
 			case 0x03: /* PTU600 */
 				model = &usbCintiqPartner; break;
 
+			case 0x39: /* PL710 */
+				common->wcmResolX = common->wcmResolY = 2540;
 			case 0x30: /* PL400 */
 			case 0x31: /* PL500 */
 			case 0x32: /* PL600 */
 			case 0x33: /* PL600SX */
 			case 0x34: /* PL550 */
 			case 0x35: /* PL800 */
+			case 0x37: /* PL700 */
+			case 0x38: /* PL510 */
+			case 0xC0: /* DTF720 */
 				model = &usbCintiq; break;
 
 			case 0x41: /* Intuos2 4x5 */
@@ -303,25 +368,35 @@ static Bool usbInit(LocalDevicePtr local)
 			case 0x60: /* Volito */
 				model = &usbVolito; break;
 
+			case 0x61: /* PenStation */
+			case 0x62: /* Volito2 4x5 */
+			case 0x63: /* Volito2 2x3 */
+			case 0x64: /* Volito2 2x3 */
+				model = &usbVolito2; break;
+
 			case 0xB0: /* Intuos3 4x5 */
 			case 0xB1: /* Intuos3 6x8 */
 			case 0xB2: /* Intuos3 9x12 */
+			case 0xB5: /* Intuos3 6x11 */
 				model = &usbIntuos3; break;
+
+			case 0x3F: /* Cintiq 21UX */
+				model = &usbCintiqV5; break;
 		}
 	}
 
 	if (!model)
 		model = &usbUnknown;
 
-	return xf86WcmInitTablet(common,model,local->fd,id,0.0);
+	return xf86WcmInitTablet(local,model,id,0.0);
 }
 
-static void usbInitProtocol5(WacomCommonPtr common, int fd, const char* id,
+static void usbInitProtocol5(WacomCommonPtr common, const char* id,
 	float version)
 {
 	DBG(2, ErrorF("detected a protocol 5 model (%s)\n",id));
 	common->wcmProtocolLevel = 5;
-	if ( strstr(id, "Intuos3") )
+	if ( strstr(id, "Intuos3") || strstr(id, "21UX") )
 	{
 		common->wcmChannelCnt = 1;
 		common->wcmResolX = common->wcmResolY = 5080;
@@ -334,11 +409,21 @@ static void usbInitProtocol5(WacomCommonPtr common, int fd, const char* id,
 	common->wcmPktLength = sizeof(struct input_event);
 }
 
-static void usbInitProtocol4(WacomCommonPtr common, int fd, const char* id,
+static void usbInitProtocol4(WacomCommonPtr common, const char* id,
 	float version)
 {
 	DBG(2, ErrorF("detected a protocol 4 model (%s)\n",id));
-	common->wcmResolX = common->wcmResolY = 1016;
+	if ( !common->wcmResolX )
+	{
+		if ( strstr(id, "Cintiq") )
+			common->wcmResolX = common->wcmResolY = 508;
+		else if ( strstr(id, "PenPartner") )
+			common->wcmResolX = common->wcmResolY = 1000;
+		else if ( strstr(id, "Graphire") )
+			common->wcmResolX = common->wcmResolY = 2032;
+		else
+			common->wcmResolX = common->wcmResolY = 1016;
+	}
 	common->wcmProtocolLevel = 4;
 	common->wcmPktLength = sizeof(struct input_event);
 }
@@ -350,13 +435,14 @@ static void usbInitProtocol4(WacomCommonPtr common, int fd, const char* id,
 #define OFF(x)   ((x)%BITS_PER_LONG)
 #define LONG(x)  ((x)/BITS_PER_LONG)
 
-static int usbGetRanges(WacomCommonPtr common, int fd)
+static int usbGetRanges(LocalDevicePtr local)
 {
 	int nValues[5];
 	unsigned long ev[NBITS(EV_MAX)];
 	unsigned long abs[NBITS(ABS_MAX)];
+	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
 
-	if (ioctl(fd, EVIOCGBIT(0 /*EV*/, sizeof(ev)), ev) < 0)
+	if (ioctl(local->fd, EVIOCGBIT(0 /*EV*/, sizeof(ev)), ev) < 0)
 	{
 		ErrorF("WACOM: unable to ioctl event bits.\n");
 		return !Success;
@@ -365,7 +451,7 @@ static int usbGetRanges(WacomCommonPtr common, int fd)
 	/* absolute values */
 	if (ISBITSET(ev,EV_ABS))
 	{
-		if (ioctl(fd, EVIOCGBIT(EV_ABS,sizeof(abs)),abs) < 0)
+		if (ioctl(local->fd, EVIOCGBIT(EV_ABS,sizeof(abs)),abs) < 0)
 		{
 			ErrorF("WACOM: unable to ioctl abs bits.\n");
 			return !Success;
@@ -374,7 +460,7 @@ static int usbGetRanges(WacomCommonPtr common, int fd)
 		/* max x */
 		if (common->wcmMaxX == 0)
 		{
-			if (ioctl(fd, EVIOCGABS(ABS_X), nValues) < 0)
+			if (ioctl(local->fd, EVIOCGABS(ABS_X), nValues) < 0)
 			{
 				ErrorF("WACOM: unable to ioctl xmax value.\n");
 				return !Success;
@@ -390,7 +476,7 @@ static int usbGetRanges(WacomCommonPtr common, int fd)
 		/* max y */
 		if (common->wcmMaxY == 0)
 		{
-			if (ioctl(fd, EVIOCGABS(ABS_Y), nValues) < 0)
+			if (ioctl(local->fd, EVIOCGABS(ABS_Y), nValues) < 0)
 			{
 				ErrorF("WACOM: unable to ioctl ymax value.\n");
 				return !Success;
@@ -404,7 +490,7 @@ static int usbGetRanges(WacomCommonPtr common, int fd)
 		}
 
 		/* max z cannot be configured */
-		if (ioctl(fd, EVIOCGABS(ABS_PRESSURE), nValues) < 0)
+		if (ioctl(local->fd, EVIOCGABS(ABS_PRESSURE), nValues) < 0)
 		{
 			ErrorF("WACOM: unable to ioctl press max value.\n");
 			return !Success;
@@ -456,14 +542,21 @@ static void usbParseEvent(WacomCommonPtr common,
 	/* packet terminated by MSC_SERIAL on kernel 2.4 and SYN_REPORT on kernel 2.6 */
 	if ((event->type != EV_MSC) || (event->code != MSC_SERIAL))
 	{
+		/* 2.6 none serial number tools fall here */
 #ifdef EV_SYN
-		/* none serial number tools fall here */
 		if ((event->type == EV_SYN) && (event->code == SYN_REPORT) 
 			&& (common->wcmChannelCnt == 1))
 		{
 			usbParseChannel(common,0,0);
 			common->wcmEventCnt = 0;
 		}
+#else
+		if ((event->type == 0x00) && (event->code == 0x00) 
+			&& (common->wcmChannelCnt == 1))
+		{
+			usbParseChannel(common,0,0);
+			common->wcmEventCnt = 0;
+		}
 #endif
 		return;
 	}
@@ -474,8 +567,8 @@ static void usbParseEvent(WacomCommonPtr common,
 	/* one channel only? */
 	if (common->wcmChannelCnt == 1)
 	{
-		/* Intuos3 Pad */
-		if (serial == 0xffffffff)
+		/* Intuos3 or Graphire4 Pad */
+		if (serial == 0xffffffff || serial == 0xf0)
 		{
 			channel = 1;
 			(&common->wcmChannel[channel].work)->device_type = PAD_ID;
@@ -594,6 +687,8 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 				ds->abswheel = event->value;
 			else if (event->code == ABS_THROTTLE)
 				ds->throttle = event->value;
+			else if (event->code == ABS_MISC && event->value)
+				ds->device_id = event->value;
 		}
 		else if (event->type == EV_REL)
 		{
@@ -619,7 +714,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 			{
 				ds->device_type = ERASER_ID;
 				ds->proximity = (event->value != 0);
-				if (ds->proximity) 
+				if (ds->proximity)
 					ds->proximity = ERASER_PROX;
 				DBG(6, ErrorF("USB eraser detected %x\n",
 					event->code));
@@ -634,7 +729,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 			}
 			else if (event->code == BTN_TOOL_FINGER)
 			{
-				DBG(6, ErrorF("USB Intuos3 Pad detected %x\n",
+				DBG(6, ErrorF("USB Pad detected %x\n",
 					event->code));
 				ds->device_type = PAD_ID;
 				ds->proximity = (event->value != 0);
@@ -676,7 +771,14 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 			else if (event->code == BTN_7)
 				MOD_BUTTONS (15, event->value);
 		}
-	} /* next event */	
+	} /* next event */
+
+	/* DTF720 doesn't support eraser */
+	if (common->tablet_id == 0xC0 && ds->device_type == ERASER_ID) 
+	{
+		DBG(10, ErrorF("usbParseChannel DTF720 doesn't support eraser "));
+		return;
+	}
 
 	/* dispatch event */
 	xf86WcmEvent(common, channel, ds);
