@@ -1,7 +1,8 @@
 /* $XConsortium: xf86Wacom.c /main/20 1996/10/27 11:05:20 kaleb $ */
 /*
- * Copyright 1995-2004 by Frederic Lepied, France. <Lepied@XFree86.org>
- *                                                                            
+ * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org> 
+ * Copyright 2002-2005 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * 
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
  * the  above copyright   notice appear  in   all  copies and  that both  that
@@ -83,9 +84,16 @@
  * 2004-05-25 26-j0.6.3  - new release
  * 2004-10-05 26-j0.6.5  - new release
  * 2004-11-22 42-j0.6.6  - new release
-*/
+ * 2005-02-17 42-j0.6.7  - added 64-bit support
+ * 2005-03-10 42-j0.6.8  - added Cintiq 21UX support
+ * 2005-05-16 47-pc0.6.9 - added tablet orentation rotation for all tablets
+ * 2005-10-17 47-pc0.7.1 - Added DTU710, DTF720, G4
+ * 2005-11-17 47-pc0.7.1-1 - Report tool serial number and ID to Xinput
+ * 2005-12-02 47-pc0.7.1-2 - Grap the USB port so /dev/input/mice won't get it
+ * 2005-12-21 47-pc0.7.2 - new release
+ */
 
-static const char identification[] = "$Identification: 42-j0.6.6 $";
+static const char identification[] = "$Identification: 47-0.7.2 $";
 
 /****************************************************************************/
 
@@ -337,8 +345,8 @@ static int xf86WcmDevOpen(DeviceIntPtr pWcm)
 		tabletSize = priv->bottomX - priv->topX;
 
 	InitValuatorAxisStruct(pWcm, 0, 0, tabletSize, /* max val */
-		mils(common->wcmResolX), /* tablet resolution */
-		0, mils(common->wcmResolX)); /* max_res */
+		common->wcmResolX, /* tablet resolution */
+		0, common->wcmResolX); /* max_res */
 
 	if (priv->twinview == TV_ABOVE_BELOW)
 		tabletSize = 2*(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
@@ -346,8 +354,8 @@ static int xf86WcmDevOpen(DeviceIntPtr pWcm)
 		tabletSize = priv->bottomY - priv->topY;
 
 	InitValuatorAxisStruct(pWcm, 1, 0, tabletSize, /* max val */
-		mils(common->wcmResolY), /* tablet resolution */
-		0, mils(common->wcmResolY)); /* max_res */
+		common->wcmResolY, /* tablet resolution */
+		0, common->wcmResolY); /* max_res */
 
 	/* pressure */
 	InitValuatorAxisStruct(pWcm, 2, 0,
@@ -378,8 +386,10 @@ static int xf86WcmDevOpen(DeviceIntPtr pWcm)
 		/* Intuos3 Marker Pen rotation */
 		InitValuatorAxisStruct(pWcm, 5, -900, 899, 1, 1, 1);
 	else
+	{
 		/* absolute wheel */
 		InitValuatorAxisStruct(pWcm, 5, 0, 1023, 1, 1, 1);
+	}
 	return TRUE;
 }
 
@@ -538,20 +548,8 @@ static int xf86WcmDevProc(DeviceIntPtr pWcm, int what)
 		case DEVICE_INIT: 
 			DBG(1, ErrorF("xf86WcmProc pWcm=%p what=INIT\n", (void *)pWcm));
 
-			nbaxes = 6;  /* X, Y, Pressure, Tilt-X, Tilt-Y, Wheel */
-
-			switch(DEVICE_ID(priv->flags))
-			{
-				case ERASER_ID:
-					nbbuttons = 1;
-					break;
-				case STYLUS_ID:
-					nbbuttons = 4;
-					break;
-				default:
-					nbbuttons = 16;
-					break;
-			}
+			nbaxes = 8;  /* X, Y, Pressure, Tilt-X, Tilt-Y, Wheel, tool ID, and serial number */
+			nbbuttons = 16;  /* All tools can report up to 16 buttons */ 
 
 			for(loop=1; loop<=nbbuttons; loop++) map[loop] = loop;
 
@@ -649,6 +647,7 @@ static int xf86WcmDevProc(DeviceIntPtr pWcm, int what)
 		default:
 			ErrorF("wacom unsupported mode=%d\n", what);
 			return !Success;
+			break;
 	} /* end switch */
 
 	DBG(2, ErrorF("END xf86WcmProc Success what=%d dev=%p priv=%p\n",
@@ -792,12 +791,27 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		break;
 	    case XWACOM_PARAM_GIMP:
 		if ((value != 0) && (value != 1)) return BadValue;
-		priv->common->wcmGimp = 0;
-		if (value) priv->common->wcmGimp = value;
+		priv->common->wcmGimp = value;
+		if (value)
+		{
+			xf86ReplaceStrOption(local->options, "Gimp", "on");
+		}
+		else
+		{
+			xf86ReplaceStrOption(local->options, "Gimp", "off");
+		}
 		break;
 	    case XWACOM_PARAM_MMT:
 		if ((value != 0) && (value != 1)) return BadValue;
 		priv->common->wcmMMonitor = value;
+		if (value)
+		{
+			xf86ReplaceStrOption(local->options, "MMonitor", "on");
+		}
+		else
+		{
+			xf86ReplaceStrOption(local->options, "MMonitor", "off");
+		}
 		break;
 	    case XWACOM_PARAM_TPCBUTTON:
 		if ((value != 0) && (value != 1)) return BadValue;
@@ -917,6 +931,20 @@ static int xf86WcmOptionCommandToFile(LocalDevicePtr local)
 				fprintf(fp, "xsetwacom set %s ClickForce %d\n", local->name, value);
 		}
 
+		s = xf86FindOptionValue(local->options, "Gimp");
+		if ( s )
+		{
+			if ( !priv->common->wcmGimp )
+				fprintf(fp, "xsetwacom set %s Gimp off\n", local->name);
+		}
+
+		s = xf86FindOptionValue(local->options, "MMonitor");
+		if ( s )
+		{
+			if ( !priv->common->wcmMMonitor )
+				fprintf(fp, "xsetwacom set %s MMonitor off\n", local->name);
+		}
+
 		s = xf86FindOptionValue(local->options, "ForceDevice");
 		if ( s )
 		{
@@ -942,6 +970,8 @@ static int xf86WcmOptionCommandToFile(LocalDevicePtr local)
 		fprintf(fp, "default SpeedLevel 6\n");
 		fprintf(fp, "default ClickForce 6\n");
 		fprintf(fp, "default Accel 0\n");
+		fprintf(fp, "default Gimp on\n");
+		fprintf(fp, "default Mmonitor on\n");
 		fclose(fp);
 	}
 	return(Success);
@@ -1072,7 +1102,7 @@ static int xf86WcmDevChangeControl(LocalDevicePtr local, xDeviceCtl* control)
 		}
 		case 4: /* JEJ - test */
 		{
-			DBG(10,ErrorF("xf86WcmChangeControl: %x,%x\n",
+			DBG(10,ErrorF("xf86WcmChangeControl: 0x%x,0x%x\n",
 				param,value));
 			return xf86WcmSetParam(local,param,value);
 		}
@@ -1085,7 +1115,7 @@ static int xf86WcmDevChangeControl(LocalDevicePtr local, xDeviceCtl* control)
 
 /*****************************************************************************
  * xf86WcmDevConvert --
- *   Convert valuators to X and Y. Only used by core events.
+ 
  ****************************************************************************/
 
 static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
@@ -1095,107 +1125,130 @@ static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
     
 	DBG(6, ErrorF("xf86WcmDevConvert v0=%d v1=%d \n", v0, v1));
 
-	if (first != 0 || num == 1)
-		return FALSE;
-#ifdef PANORAMIX
-	if (!noPanoramiXExtension && (priv->flags & ABSOLUTE_FLAG) && 
-		priv->common->wcmGimp && priv->common->wcmMMonitor)
-	{
-		int i, totalWidth, leftPadding = 0;
-		for (i = 0; i < priv->currentScreen; i++)
-			leftPadding += screenInfo.screens[i]->width;
-		for (totalWidth = leftPadding; i < priv->numScreen; i++)
-			totalWidth += screenInfo.screens[i]->width;
-		v0 -= (priv->bottomX - priv->topX) * leftPadding
-			/ (double)totalWidth + 0.5;
-	}
-#endif
-	if (priv->twinview != TV_NONE && (priv->flags & ABSOLUTE_FLAG))
-	{
-		v0 -= priv->topX - priv->tvoffsetX;
-		v1 -= priv->topY - priv->tvoffsetY;
-                if (priv->twinview == TV_LEFT_RIGHT)
-		{
-			if (v0 > priv->bottomX - priv->tvoffsetX)
-			{
-				if (priv->currentScreen == 0)
-					v0 = priv->bottomX - priv->tvoffsetX;
-				else
-				{
-					v0 -= priv->bottomX - priv->topX - 2*priv->tvoffsetX;
-					if (v0 > priv->bottomX - priv->tvoffsetX)
-						v0 = 2*(priv->bottomX - priv->tvoffsetX) - v0;
-				}
-			}
-			if (priv->currentScreen == 1)
-			{
-                       		*x = priv->tvResolution[0] + priv->tvResolution[2]
-					* v0 / (priv->bottomX - priv->topX - 2*priv->tvoffsetX);
-				*y = v1 * priv->tvResolution[3] /
-					(priv->bottomY - priv->topY - 2*priv->tvoffsetY) + 0.5;
-			}
-			else
-			{
-				*x = priv->tvResolution[0] * v0 
-					 / (priv->bottomX - priv->topX - 2*priv->tvoffsetX);
-				*y = v1 * priv->tvResolution[1] /
-					(priv->bottomY - priv->topY - 2*priv->tvoffsetY) + 0.5;
-			}
-		}
-                if (priv->twinview == TV_ABOVE_BELOW)
-		{
-			if (v1 > priv->bottomY - priv->tvoffsetY)
-			{
-				if (priv->currentScreen == 0)
-					v1 = priv->bottomY - priv->tvoffsetY;
-				else
-				{
-					v1 -= priv->bottomY - priv->topY - 2*priv->tvoffsetY;
-					if (v1 > priv->bottomY - priv->tvoffsetY)
-						v1 = 2*(priv->bottomY - priv->tvoffsetY) - v1;
-				}
-			}
-			if (priv->currentScreen == 1)
-			{
- 				*x = v0 * priv->tvResolution[2] /
-					(priv->bottomX - priv->topX - 2*priv->tvoffsetX) + 0.5;
-				*y = priv->tvResolution[1] + 
-					priv->tvResolution[3] * v1 / 
-					(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
-			}
-			else
-			{
-				*x = v0 * priv->tvResolution[0] /
-					(priv->bottomX - priv->topX - 2*priv->tvoffsetX) + 0.5;
-				*y = priv->tvResolution[1] * v1 /
-					 (priv->bottomY - priv->topY - 2*priv->tvoffsetY);
-			}
-		}
-	}
-	else
-	{
-		*x = v0 * priv->factorX + 0.5;
-		*y = v1 * priv->factorY + 0.5;
-	}
+	if (first != 0 || num == 1) 
+ 		return FALSE;
 
-	DBG(6, ErrorF("Wacom converted v0=%d v1=%d to x=%d y=%d\n",
-		v0, v1, *x, *y));
+	*x = 0;
+	*y = 0;
+
+	if (priv->flags & ABSOLUTE_FLAG)
+	{
+		if (priv->twinview == TV_NONE)
+		{
+			v0 = v0 > priv->bottomX ? priv->bottomX - priv->topX : 
+				v0 < priv->topX ? 0 : v0 - priv->topX;
+			v1 = v1 > priv->bottomY ? priv->bottomY - priv->topY : 
+				v1 < priv->topY ? 0 : v1 - priv->topY;
+		}
+
+#ifdef PANORAMIX
+		if (!noPanoramiXExtension && priv->common->wcmGimp 
+			&& priv->common->wcmMMonitor)
+		{
+			int i, totalWidth, leftPadding = 0;
+			for (i = 0; i < priv->currentScreen; i++)
+				leftPadding += screenInfo.screens[i]->width;
+			for (totalWidth = leftPadding; i < priv->numScreen; i++)
+				totalWidth += screenInfo.screens[i]->width;
+			v0 -= (priv->bottomX - priv->topX) * leftPadding
+				/ (double)totalWidth + 0.5;
+		}
+#endif
+		if (priv->twinview != TV_NONE)
+		{
+			v0 -= priv->topX - priv->tvoffsetX;
+			v1 -= priv->topY - priv->tvoffsetY;
+               		if (priv->twinview == TV_LEFT_RIGHT)
+			{
+				if (v0 > priv->bottomX - priv->tvoffsetX && priv->screen_no == -1)
+				{
+					if (priv->currentScreen == 0)
+						v0 = priv->bottomX - priv->tvoffsetX;
+					else
+					{
+						v0 -= priv->bottomX - priv->topX - 2*priv->tvoffsetX;
+						if (v0 > priv->bottomX - priv->tvoffsetX)
+							v0 = 2*(priv->bottomX - priv->tvoffsetX) - v0;
+					}
+				}
+				if (priv->currentScreen == 1)
+				{
+                       			*x = priv->tvResolution[0] + priv->tvResolution[2]
+						* v0 / (priv->bottomX - priv->topX - 2*priv->tvoffsetX);
+					*y = v1 * priv->tvResolution[3] /
+						(priv->bottomY - priv->topY - 2*priv->tvoffsetY) + 0.5;
+				}
+				else
+				{
+					*x = priv->tvResolution[0] * v0 
+						 / (priv->bottomX - priv->topX - 2*priv->tvoffsetX);
+					*y = v1 * priv->tvResolution[1] /
+						(priv->bottomY - priv->topY - 2*priv->tvoffsetY) + 0.5;
+				}
+			}
+            		if (priv->twinview == TV_ABOVE_BELOW)
+			{
+				if (v1 > priv->bottomY - priv->tvoffsetY && priv->screen_no == -1)
+				{
+					if (priv->currentScreen == 0)
+						v1 = priv->bottomY - priv->tvoffsetY;
+					else
+					{
+						v1 -= priv->bottomY - priv->topY - 2*priv->tvoffsetY;
+						if (v1 > priv->bottomY - priv->tvoffsetY)
+							v1 = 2*(priv->bottomY - priv->tvoffsetY) - v1;
+					}
+				}
+				if (priv->currentScreen == 1)
+				{
+ 					*x = v0 * priv->tvResolution[2] /
+						(priv->bottomX - priv->topX - 2*priv->tvoffsetX) + 0.5;
+					*y = priv->tvResolution[1] + 
+						priv->tvResolution[3] * v1 / 
+						(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
+				}
+				else
+				{
+					*x = v0 * priv->tvResolution[0] /
+						(priv->bottomX - priv->topX - 2*priv->tvoffsetX) + 0.5;
+					*y = priv->tvResolution[1] * v1 /
+						(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
+				}
+			}
+			return TRUE;
+		}
+	}
+	*x += v0 * priv->factorX + 0.5;
+	*y += v1 * priv->factorY + 0.5;
+
+	DBG(6, ErrorF("Wacom converted v0=%d v1=%d to x=%d y=%d\n", v0, v1, *x, *y));
 	return TRUE;
 }
 
 /*****************************************************************************
  * xf86WcmDevReverseConvert --
  *  Convert X and Y to valuators. Only used by core events.
+ *  Handdle relatve screen to tablet convert
+ *  We don't support screen mapping yet (Ping April 22, 2005)
  ****************************************************************************/
 
 static Bool xf86WcmDevReverseConvert(LocalDevicePtr local, int x, int y,
 		int* valuators)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-    
-	DBG(6, ErrorF("xf86WcmDevReverseConvert x=%d y=%d \n", x, y));
-	valuators[0] = x / priv->factorX + 0.5;
-	valuators[1] = y / priv->factorY + 0.5;
+
+ 	DBG(6, ErrorF("xf86WcmDevReverseConvert x=%d y=%d \n", x, y));
+	priv->currentSX = x;
+	priv->currentSY = y;
+
+	if (!(priv->flags & ABSOLUTE_FLAG))
+	{
+		valuators[0] = x / priv->factorX + 0.5;
+		valuators[1] = y / priv->factorY + 0.5;
+	}
+
+#ifdef NEVER 
+/* This is for absolute screen mapping */
 #ifdef PANORAMIX
 	if (!noPanoramiXExtension && (priv->flags & ABSOLUTE_FLAG) && 
 		priv->common->wcmGimp && priv->common->wcmMMonitor)
@@ -1218,8 +1271,7 @@ static Bool xf86WcmDevReverseConvert(LocalDevicePtr local, int x, int y,
 			if (priv->currentScreen == 1)
 			{
 				valuators[0] = x * (priv->bottomX - priv->topX - 2*priv->tvoffsetX)
-				 	/ priv->tvResolution[2] + (priv->bottomX - priv->topX - 
-					2*priv->tvoffsetX) +0.5;
+				 	/ priv->tvResolution[2] + 0.5;
 				valuators[1] = y * (priv->bottomY - priv->topY - 2*priv->tvoffsetY) /
 					priv->tvResolution[3] + 0.5;
 			}
@@ -1240,8 +1292,7 @@ static Bool xf86WcmDevReverseConvert(LocalDevicePtr local, int x, int y,
 				valuators[0] = x * (priv->bottomX - priv->topX - 2*priv->tvoffsetX) /
 					priv->tvResolution[2] + 0.5;
 				valuators[1] = y *(priv->bottomY - priv->topY - 2*priv->tvoffsetY)
-					/ priv->tvResolution[3] +
-					priv->bottomY - priv->topY - 2*priv->tvoffsetY + 0.5;
+					/ priv->tvResolution[3] + 0.5;
 			}
 			else
 			{
@@ -1251,11 +1302,15 @@ static Bool xf86WcmDevReverseConvert(LocalDevicePtr local, int x, int y,
 					/ priv->tvResolution[1] + 0.5;
 			}
 		}
+	}
+	if (priv->flags & ABSOLUTE_FLAG)
+	{
 		valuators[0] += priv->topX + priv->tvoffsetX;
 		valuators[1] += priv->topY + priv->tvoffsetY;
 	}
-	DBG(6, ErrorF("Wacom converted x=%d y=%d to v0=%d v1=%d\n", x, y,
-		valuators[0], valuators[1]));
+#endif
+	DBG(6, ErrorF("Wacom converted x=%d y=%d to v0=%d v1=%d v2=%d v3=%d v4=%d v5=%d\n", x, y,
+		valuators[0], valuators[1], valuators[2], valuators[3], valuators[4], valuators[5]));
 
 	return TRUE;
 }
