@@ -1,6 +1,7 @@
 /*
- * Copyright 1995-2004 by Frederic Lepied, France. <Lepied@XFree86.org>
- *                                                                            
+ * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
+ * Copyright 2002-2005 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * 
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is  hereby granted without fee, provided that
  * the  above copyright   notice appear  in   all  copies and  that both  that
@@ -9,8 +10,8 @@
  * advertising or publicity pertaining to distribution of the software without
  * specific,  written      prior  permission.     Frederic  Lepied   makes  no
  * representations about the suitability of this software for any purpose.  It
- * is provided "as is" without express or implied warranty.                   
- *                                                                            
+ * is provided "as is" without express or implied warranty.
+ * 
  * FREDERIC  LEPIED DISCLAIMS ALL   WARRANTIES WITH REGARD  TO  THIS SOFTWARE,
  * INCLUDING ALL IMPLIED   WARRANTIES OF MERCHANTABILITY  AND   FITNESS, IN NO
  * EVENT  SHALL FREDERIC  LEPIED BE   LIABLE   FOR ANY  SPECIAL, INDIRECT   OR
@@ -190,8 +191,7 @@ struct _WacomModule
 	int (*DevChangeControl)(LocalDevicePtr local, xDeviceCtl* control);
 	int (*DevSwitchMode)(ClientPtr client, DeviceIntPtr dev, int mode);
 	Bool (*DevConvert)(LocalDevicePtr local, int first, int num,
-		int v0, int v1, int v2, int v3, int v4, int v5,
-		int* x, int* y);
+		int v0, int v1, int v2, int v3, int v4, int v5, int* x, int* y);
 	Bool (*DevReverseConvert)(LocalDevicePtr local, int x, int y,
 		int* valuators);
 };
@@ -206,14 +206,14 @@ struct _WacomModel
 {
 	const char* name;
 
-	void (*Initialize)(WacomCommonPtr common, int fd, const char* id, float version);
-	void (*GetResolution)(WacomCommonPtr common, int fd);
-	int (*GetRanges)(WacomCommonPtr common, int fd);
-	int (*Reset)(WacomCommonPtr common, int fd);
-	int (*EnableTilt)(WacomCommonPtr common, int fd);
-	int (*EnableSuppress)(WacomCommonPtr common, int fd);
-	int (*SetLinkSpeed)(WacomCommonPtr common, int fd);
-	int (*Start)(WacomCommonPtr common, int fd);
+	void (*Initialize)(WacomCommonPtr common, const char* id, float version);
+	void (*GetResolution)(LocalDevicePtr local);
+	int (*GetRanges)(LocalDevicePtr local);
+	int (*Reset)(LocalDevicePtr local);
+	int (*EnableTilt)(LocalDevicePtr local);
+	int (*EnableSuppress)(LocalDevicePtr local);
+	int (*SetLinkSpeed)(LocalDevicePtr local);
+	int (*Start)(LocalDevicePtr local);
 	int (*Parse)(WacomCommonPtr common, const unsigned char* data);
 	int (*FilterRaw)(WacomCommonPtr common, WacomChannelPtr pChannel,
 		WacomDeviceStatePtr ds);
@@ -224,6 +224,9 @@ struct _WacomModel
  *****************************************************************************/
 
 #define DEVICE_ID(flags) ((flags) & 0x0f)
+#define STYLUS_DEVICE_ID	0x02
+#define CURSOR_DEVICE_ID	0x06
+#define ERASER_DEVICE_ID	0x0A
 
 #define STYLUS_ID               1
 #define CURSOR_ID               2
@@ -268,6 +271,10 @@ struct _WacomDeviceRec
 	WacomCommonPtr common;  /* common info pointer */
     
 	/* state fields */
+	int currentX;           /* current X position */
+	int currentY;           /* current Y position */
+	int currentSX;          /* current screen X position */
+	int currentSY;          /* current screen Y position */
 	int oldX;               /* previous X position */
 	int oldY;               /* previous Y position */
 	int oldZ;               /* previous pressure */
@@ -320,7 +327,7 @@ struct _WacomDeviceRec
 
 struct _WacomDeviceState
 {
-	int device_id;
+	int device_id;		/* tool id reported from the physical device */
 	int device_type;
 	unsigned int serial_num;
 	int x;
@@ -419,6 +426,7 @@ struct _WacomCommonRec
 	char* wcmDevice;             /* device file name */
 	int wcmSuppress;             /* transmit position on delta > supress */
 	unsigned char wcmFlags;      /* various flags (handle tilt) */
+	int tablet_id;		     /* USB tablet ID */
 
 	/* These values are in tablet coordinates */
 	int wcmMaxX;                 /* tablet max X value */
@@ -454,12 +462,12 @@ struct _WacomCommonRec
 	char * wcmEraserID;	     /* eraser associated with the stylus */
 	int wcmGimp;                 /* support Gimp on Xinerama Enabled multi-monitor desktop */
 	int wcmMMonitor;             /* disable/enable moving across screens in multi-monitor desktop */
-	int wcmTPCButton;		/* set Tablet PC button on/off */
+	int wcmTPCButton;	     /* set Tablet PC button on/off */
 
 	int bufpos;                        /* position with buffer */
 	unsigned char buffer[BUFFER_SIZE]; /* data read from device */
 
-#if LINUX_INPUT
+#ifdef LINUX_INPUT
 	int wcmEventCnt;
 	struct input_event wcmEvents[MAX_USB_EVENTS];  /* events for current change */
 #endif
@@ -501,23 +509,21 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag);
 LocalDevicePtr xf86WcmAllocateStylus(void);
 LocalDevicePtr xf86WcmAllocateCursor(void);
 LocalDevicePtr xf86WcmAllocateEraser(void);
+LocalDevicePtr xf86WcmAllocatePad(void);
 
 Bool xf86WcmOpen(LocalDevicePtr local);
 
-	int xf86WcmInitTablet(WacomCommonPtr common, WacomModelPtr model,
-		int fd, const char* id, float version);
-	/* common tablet initialization regime */
+/* common tablet initialization regime */
+int xf86WcmInitTablet(LocalDevicePtr local, WacomModelPtr model, const char* id, float version);
 
-	void xf86WcmReadPacket(LocalDevicePtr local);
-	/* standard packet handler */
+/* standard packet handler */
+void xf86WcmReadPacket(LocalDevicePtr local);
 
-	void xf86WcmEvent(WacomCommonPtr common,
-		unsigned int channel, const WacomDeviceState* ds);
-	/* handles suppression, filtering, and dispatch. */
+/* handles suppression, filtering, and dispatch. */
+void xf86WcmEvent(WacomCommonPtr common, unsigned int channel, const WacomDeviceState* ds);
 
-	void xf86WcmSendEvents(LocalDevicePtr local,
-		const WacomDeviceState* ds, unsigned int channel);
-	/* dispatches data to XInput event system */
+/* dispatches data to XInput event system */
+void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds, unsigned int channel);
 
 /****************************************************************************/
 #endif /* __XF86WACOM_H */
