@@ -2,6 +2,7 @@
 ** xidump.c
 **
 ** Copyright (C) 2003 - 2004 - John E. Joganic
+** Copyright (C) 2004 - 2005 - Ping Cheng 
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -24,6 +25,8 @@
 **   2003-03-21 0.0.4 - added conditional curses code
 **   2003-03-21 0.5.0 - released in development branch
 **   2003-04-07 0.5.1 - added pressure bar
+**   2005-07-27 0.5.2 - remove unused GTK stuff [jg]
+**   2005-11-11 0.7.1 - report tool ID and serial number
 **
 ****************************************************************************/
 
@@ -35,7 +38,7 @@
 #include <sys/time.h>
 #include <math.h>
 
-#define XIDUMP_VERSION "0.5.1"
+#define XIDUMP_VERSION "0.7.1"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -204,7 +207,7 @@ int ListDevices(Display* pDisp, const char* pszDeviceName)
 					}
 	
 					default:
-						printf("  unk: class=%lu\n",pClass->class);
+						printf("  unknown class\n" );
 				}
 	
 				/* skip to next record */
@@ -365,33 +368,6 @@ struct _UI
 	int (*Run)(Display* pDisp, XDeviceInfo* pDevInfo, FORMATTYPE fmt);
 };
 
-/*****************************************************************************
-** GTK UI
-*****************************************************************************/
-
-#if WCM_ENABLE_GTK12 || WCM_ENABLE_GTK20
-#define USE_GTK 1
-#include <gtk/gtk.h>
-
-static int GTKInit(void)
-{
-	return 1;
-}
-
-static void GTKTerm(void)
-{
-}
-
-static int GTKRun(Display* pDisp, XDeviceInfo* pDevInfo, FORMATTYPE fmt)
-{
-	return 1;
-}
-
-	UI gGTKUI = { "gtk", GTKInit, GTKTerm, GTKRun };
-#else
-#define USE_GTK 0
-#endif
-
 /*****************************************************************************
 ** Curses UI
 *****************************************************************************/
@@ -425,7 +401,7 @@ static int CursesInit(void)
 static int CursesRun(Display* pDisp, XDeviceInfo* pDevInfo, FORMATTYPE fmt)
 {
 	int i, j, k, bDown, nBtn;
-	int nRow=0, nPressRow, nProxRow, nFocusRow, nButtonRow,
+	int nRow=0, nTitleRow, nPressRow, nProxRow, nFocusRow, nButtonRow,
        		nKeyRow, nValRow;
 	int nMaxPress=100, nMinPress=0;
 	int bStylus = 0;
@@ -462,13 +438,12 @@ static int CursesRun(Display* pDisp, XDeviceInfo* pDevInfo, FORMATTYPE fmt)
 	nRow += 1;
 
 	/* display valuator related info */
+	nTitleRow = nRow;
 	if (pValInfo)
 	{
-		snprintf(chBuf,sizeof(chBuf),"Valuators: %s   Axes: %d  Buffer: %ld",
+		snprintf(chBuf,sizeof(chBuf),"Valuators: %s   ID: Undefined  Serial Number: Undefined",
 				pValInfo->mode == Absolute ? "Absolute" :
-				pValInfo->mode == Relative ? "Relative" : "Unknown",
-				pValInfo->num_axes,
-				pValInfo->motion_buffer);
+				pValInfo->mode == Relative ? "Relative" : "Unknown");
 		wacscrn_output(nRow,0,chBuf);
 		nRow += 2;
 		nValRow = nRow;
@@ -536,6 +511,10 @@ static int CursesRun(Display* pDisp, XDeviceInfo* pDevInfo, FORMATTYPE fmt)
 
 		if (pAny->type == gnInputEvent[INPUTEVENT_PROXIMITY_IN])
 		{
+			if (!pValInfo)
+			{
+				wacscrn_output(23,0,"Unexpected valuator data received.");
+			}
 			wacscrn_standout();
 			wacscrn_output(nProxRow,12,"IN ");
 			wacscrn_normal();
@@ -558,10 +537,36 @@ static int CursesRun(Display* pDisp, XDeviceInfo* pDevInfo, FORMATTYPE fmt)
 				wacscrn_output(23,0,"Unexpected valuator data received.");
 			}
 
-			for (k=0; k<pValInfo->num_axes && k<6; ++k)
+			/* title value */
 			{
-				snprintf(chBuf,sizeof(chBuf),"%+06d",pMove->axis_data[k]);
-				wacscrn_output(nValRow+1,12 + k * 10, chBuf);
+				int v = (pMove->axis_data[3]&0xffff0000) >> 16;
+				snprintf(chBuf,sizeof(chBuf),"%s",
+					pValInfo->mode == Absolute ? "Absolute" :
+					pValInfo->mode == Relative ? "Relative" : "Unknown");
+				wacscrn_output(nTitleRow,11,chBuf);
+				snprintf(chBuf, sizeof(chBuf), "%10d", v);
+				wacscrn_output(nTitleRow, 25, chBuf);
+				v = (pMove->axis_data[4]&0xffff0000) | 
+							((pMove->axis_data[5]&0xffff0000)>>16);
+				if ( v )
+				{
+					snprintf(chBuf,sizeof(chBuf), "%12d", v);
+					wacscrn_output(nTitleRow,52,chBuf);
+				}
+			}
+
+			for (k=0; k<pValInfo->num_axes && k<3; ++k)
+			{
+				snprintf(chBuf, sizeof(chBuf), "%+06d", pMove->axis_data[k]);
+				wacscrn_output(nValRow+1, 12 + k * 10, chBuf);
+
+			}
+
+			for (k=3; k<pValInfo->num_axes && k<6; ++k)
+			{
+				snprintf(chBuf, sizeof(chBuf), "%+06d", 
+						(short)(pMove->axis_data[k]&0xffff));
+				wacscrn_output(nValRow+1, 12 + k * 10, chBuf);
 
 			}
 
@@ -673,13 +678,20 @@ static int RawRunDefault(Display* pDisp, XDeviceInfo* pDevInfo)
 		else if (pAny->type == gnInputEvent[INPUTEVENT_MOTION_NOTIFY])
 		{
 			XDeviceMotionEvent* pMove = (XDeviceMotionEvent*)pAny;
-			printf("Motion: x=%+6d y=%+6d p=%4d tx=%+4d ty=%+4d w=%+5d\n",
+			int v = (pMove->axis_data[4]&0xffff0000) | 
+					((pMove->axis_data[5]&0xffff0000)>>16);
+
+			printf("Motion: x=%+6d y=%+6d p=%4d tx=%+4d ty=%+4d "
+				"w=%+5d ID: %4d Serial: %11d \n",
 					pMove->axis_data[0],
 					pMove->axis_data[1],
 					pMove->axis_data[2],
-					pMove->axis_data[3],
-					pMove->axis_data[4],
-					pMove->axis_data[5]);
+					(short)(pMove->axis_data[3]&0xffff),
+					(short)(pMove->axis_data[4]&0xffff),
+					(short)(pMove->axis_data[5]&0xffff),
+					(pMove->axis_data[3]&0xffff0000)>>16,
+					v);
+
 		}
 		else if ((pAny->type == gnInputEvent[INPUTEVENT_BTN_PRESS]) ||
 				(pAny->type == gnInputEvent[INPUTEVENT_BTN_RELEASE]))
@@ -794,11 +806,6 @@ static int RawRun(Display* pDisp, XDeviceInfo* pDevInfo, FORMATTYPE fmt)
 
 	UI* gpUIs[] =
 	{
-		/* GTK UI */
-		#if USE_GTK
-	/*	&gGTKUI, */ /* not ready yet */
-		#endif
-
 		/* Curses UI */
 		#if USE_NCURSES
 		&gCursesUI,
@@ -873,10 +880,10 @@ int Run(Display* pDisp, UI* pUI, FORMATTYPE fmt, const char* pszDeviceName)
 	/* create a window to receive events */
 	wnd = XCreateWindow(pDisp,
 			DefaultRootWindow(pDisp), /* parent */
-			0,0,100,100, /* placement */
+			0,0,10,10, /* placement */
 			0, /* border width */
 			0, /* depth */
-			InputOnly, /* class */
+			InputOutput, /* class */
 			CopyFromParent, /* visual */
 			0, /* valuemask */
 			NULL); /* attributes */
